import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Point;

/**
 * A Button
 * 
 * @author Taylor Born
 * @version November 2010
 */
public class Button  extends Actor
{
    private Point size;
    private String text;
    private boolean hover;
    private boolean pressed;
    private GreenfootImage icon = new GreenfootImage(1, 1);
    private boolean iconVisible;
    private boolean clicked;
    private boolean enabled = true;

    /**
     * Create a new Button.
     * @param text The text the Button is to display.
     * @param size The width and height of the Button.
     */
    public Button(String text, Point size)
    {
        this.size = size;
        this.text = text;
        draw();
    }
    
    public void act() 
    {
        if (enabled)
        {
            boolean last = hover;
            boolean lastP = pressed;
            if (Greenfoot.mouseMoved(this))
                hover = true;
            else if (Greenfoot.mouseMoved(null))
                hover = false;
            if (Greenfoot.mousePressed(this))
                pressed = true;
            if (Greenfoot.mouseClicked(this) && hover)
                clicked = true;
            if (Greenfoot.mouseClicked(null) || Greenfoot.mouseDragEnded(null))
                pressed = false;
            if (last != hover || lastP != pressed)
                draw();
        }
    }
    
    private void draw()
    {
        GreenfootImage pic = new GreenfootImage((int)size.getX(), (int)size.getY());
        if (!enabled)
            pic.setColor(new Color(102, 102, 102));
        Graphics2D graphics = (new GreenfootImage(1, 1)).getAwtImage().createGraphics();
        graphics.setFont(new Font("New Times Roman", 20, 12));
        FontMetrics fm = graphics.getFontMetrics();
        pic.setFont(new Font("New Times Roman", 20, 12));
        pic.drawString(text, ((int)size.getX() - fm.charsWidth((text).toCharArray(), 0, (text).length())) / 2, 16);
        graphics.dispose();
        if (pressed)
            pic.setColor(Color.GRAY);
        pic.drawRect(0, 0, (int)size.getX() - 1, (int)size.getY() - 1);
        if (iconVisible)
            pic.drawImage(icon, 3, 4);
        if (hover)
        {
            GreenfootImage back = new GreenfootImage((int)size.getX(), (int)size.getY());
            back.fill();
            back.setTransparency(25);
            GreenfootImage pic2 = new GreenfootImage((int)size.getX(), (int)size.getY());
            pic2.drawImage(back, 0, 0);
            pic2.drawImage(pic, 0, 0);
            pic = pic2;
        }
        setImage(pic);
    }
    
    /**
     * The action listener for the Button.
     * @return Whether the Button was clicked or not.
     */
    public boolean wasClicked()
    {
        boolean c = clicked;
        clicked = false;
        return c;
    }
    
    public void addedToWorld(World world)
    {
        hover = false;
        draw();
    }
    
    /**
     * Set if the Button is enabled.
     * @param e Whether the Button will be enabled or not.
     */
    public void setEnable(boolean e)
    {
        enabled = e;
        draw();
    }
    
    /**
     * Set if the Button is to display its icon.
     * @param show Whether the Button will display its icon.
     */
    public void showIcon(boolean show)
    {
        iconVisible = show;
        draw();
    }
    
    /**
     * Get the text the Button is displaying.
     * @return The text the Button is displaying.
     */
    public String getText()
    {
        return text;
    }
    
    /**
     * Set what text the Button is displaying.
     * @param text The text the Button will display.
     */
    public void setText(String text)
    {
        this.text = text;
        draw();
    }
}