import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.*;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Color;
import java.awt.Point;

/**
 * A DropDownList
 * 
 * @author Taylor Born
 * @version March 2011
 */
public class DropDownList  extends Actor
{
    static Font font = new Font("New Times Roman", Font.PLAIN, 12);
    
    private ArrayList<String> items = new ArrayList<String>();
    private int index = -1;
    private Point home;
    private Point lastMouse = new Point(-25, -25);
    private boolean selecting = false;
    
    private boolean changed = false;

    /**
     * Create a new DropDownList.
     * @param items The contents of the DropDownList.
     * @param index The initial selected item index.
     */
    public DropDownList(ArrayList<String> items, int index)
    {
        this.items = items;
        this.index = index;
    }
    
    public void act() 
    {
        MouseInfo mouse = Greenfoot.getMouseInfo();
        if (Greenfoot.mouseMoved(null) || Greenfoot.mouseDragged(null))
            lastMouse = new Point(mouse.getX(), mouse.getY());
        
        int height = 12;
        if (selecting)
            height += items.size() * 12;
        
        int width = 0;
        
        Graphics2D graphics = (new GreenfootImage(1, 1)).getAwtImage().createGraphics();
        graphics.setFont(DropDownList.font);
        FontMetrics fm = graphics.getFontMetrics();
        for (String s : items)
        {
            int w = fm.charsWidth(s.toCharArray(), 0, s.length());
            if (w > width)
                width = w;
        }
        width += 4;
        
        setLocation(getX(), (int)home.getY() + (height - 12) / 2);
        Point offsetMouse = new Point((int)lastMouse.getX() - (getX() - (width / 2)), (int)lastMouse.getY() - (getY() - (height / 2)));
        
        boolean clicked = Greenfoot.mouseClicked(null);
        if (Greenfoot.mouseClicked(this) && offsetMouse.getX() > 0 && offsetMouse.getX() < width && offsetMouse.getY() > 0 && offsetMouse.getY() < 12)
            selecting = !selecting;
        
        GreenfootImage pic = new GreenfootImage(width + 1, height + 1);
        pic.setFont(font);
        
        pic.setColor(Color.WHITE);
        pic.fill();
        pic.setColor(Color.BLACK);
        pic.drawRect(0, 0, width, 12);
        if (selecting)
            pic.setColor(Color.GRAY);
        if (index > -1 && index < items.size())
            pic.drawString(items.get(index), 2, 11);
        
        if (selecting)
            for (int i = 0; i < items.size(); i++)
            {
                if (offsetMouse.getX() > 0 && offsetMouse.getX() < width && offsetMouse.getY() > 12 * (i + 1) && offsetMouse.getY() <= 12 * (i + 2))
                {
                    pic.setColor(Color.CYAN);
                    pic.fillRect(0, 12 * (i + 1), width, 12);
                    if (clicked)
                    {
                        if (index != i)
                            changed = true;
                        index = i;
                        selecting = false;
                    }
                }
                pic.setColor(Color.BLACK);
                pic.drawRect(0, 12 * (i + 1), width, 12);
                pic.drawString(items.get(i), 2, (i + 1) * 12 + 11);
            }
        
        if (!Greenfoot.mouseClicked(this) && Greenfoot.mouseClicked(null))
            selecting = false;
        
        setImage(pic);
    }
    
    /**
     * @return String for the item that is selected. Null if none selected.
     */
    public String getSelected()
    {
        if (index == -1)
            return null;
        return items.get(index);
    }
    
    /**
     * @return The index of the item that is selected. -1 if none selected.
     */
    public int getIndex()
    {
        return index;
    }
    
    /**
     * @return The contents in this list.
     */
    public ArrayList<String> getList()
    {
        return items;
    }
    
    /**
     * The action listener for this DropDownList.
     * @return Whether a new item has been selected.
     */
    public boolean hasChanged()
    {
        boolean c = changed;
        changed = false;
        return c;
    }
    
    public void addedToWorld(World world)
    {
        home = new Point(getX(), getY());
    }
}