import greenfoot.*;
import java.util.*;
import java.awt.Point;
import java.awt.Color;
import java.awt.RenderingHints;
import java.awt.Graphics2D;

/**
 * Write a description of class Layer here.
 * 
 * @author Taylor Born
 * @version November 2011
 */
public class Layer  
{
    private GreenfootImage image;
    private Point position;
    //private String name;
    private boolean visible = true;
    
    public Layer(Point p, int brushSize, Color c)
    {
        position = new Point((int)p.getX() - brushSize / 2, (int)p.getY() - brushSize / 2);
        image = new GreenfootImage(brushSize, brushSize);
        image.setColor(c);
        if (brushSize == 1)
            image.fill();
        else
            image.fillOval(-1, -1, brushSize + 1, brushSize + 1);
    }
    public Layer(GreenfootImage im, Point pos)
    {
        image = im;
        position = pos;
    }
    public Layer(Layer l)
    {
        image = new GreenfootImage(l.getImage());
        position = new Point(l.getPosition());
    }
    public GreenfootImage getImage()
    {
        return image;
    }
    
    public void drawBetween(Point a, Point b, int brushSize, Color c)
    {
        drawAt((int)b.getX(), (int)b.getY(), brushSize, c);
        if (brushSize == 1)
            image.drawLine((int)(a.getX() - position.getX()), (int)(a.getY() - (int)position.getY()), (int)(b.getX() - (int)position.getX()), (int)(b.getY() - (int)position.getY()));
        else
        {
            double x = a.getX();
            double y = a.getY();
            double angle = Math.atan2(b.getY() - a.getY(), b.getX() - a.getX());
            while (true)
            {
                if (Math.abs(Math.atan2(b.getY() - y, b.getX() - x) - angle) > .01)
                    break;
                image.fillOval((int)(x - position.getX() - brushSize / 2) - 1, (int)(y - (int)position.getY() - brushSize / 2) - 1, brushSize + 1, brushSize + 1);
                x += Math.cos(angle) * (brushSize / 2d);
                y += Math.sin(angle) * (brushSize / 2d);
            }
        }
    }
    public void include(int x, int y)
    {
        int xM = x < position.getX() ? (int)position.getX() - x : x >= position.getX() + image.getWidth() ? (int)position.getX() + image.getWidth() - 1 - x : 0;
        int yM = y < position.getY() ? (int)position.getY() - y : y >= position.getY() + image.getHeight() ? (int)position.getY() + image.getHeight() - 1 - y : 0;
        if (xM != 0 || yM != 0)
        {
            GreenfootImage i = new GreenfootImage(image.getWidth() + Math.abs(xM), image.getHeight() + Math.abs(yM));
            i.drawImage(image, xM < 0 ? 0 : xM, yM < 0 ? 0 : yM);
            image = i;
            position.translate(xM > 0 ? -xM : 0, yM > 0 ? -yM : 0);
        }
    }
    public void drawAt(int x, int y, int brushSize, Color c)
    {
        boolean evenSize = brushSize % 2 == 0;
        if (brushSize == 1)
        {
            include(x, y);
            image.setColor(c);
            image.setColorAt(x - (int)position.getX(), y - (int)position.getY(), c);
        }
        else
        {
            include(x + brushSize / 2 - (evenSize ? 1 : 0), y + brushSize / 2 - (evenSize ? 1 : 0));
            include(x + brushSize / 2 + (evenSize ? 0 : 1) - brushSize, y + brushSize / 2 + (evenSize ? 0 : 1) - brushSize);
            image.setColor(c);
            image.fillOval((int)(x - (int)position.getX() - brushSize / 2) - 1, (int)(y - (int)position.getY() - brushSize / 2) - 1, brushSize + 1, brushSize + 1);
        }
    }
    private boolean[][] selected;
    private Color wandPath;
    private int tolerance;
    public boolean[][] getSelected()
    {
        return selected;
    }
    public void deselect()
    {
        selected = null;
    }
    public void selectAll()
    {
        if (selected == null)
            selected = new boolean[image.getHeight()][image.getWidth()];
        for (int i = 0; i < selected.length; i++)
            for (int k = 0; k < selected[0].length; k++)
                selected[i][k] = true;
    }
    public void wandPressedAt(int x, int y, int t)
    {
        tolerance = t;
        int xx = x - (int)position.getX();
        int yy = y - (int)position.getY();
        if (xx > -1 && xx < image.getWidth() && yy > -1 && yy < image.getHeight())
        {
            if (!Greenfoot.isKeyDown("shift") || selected == null)
                selected = new boolean[image.getHeight()][image.getWidth()];
            wandPath = image.getColorAt(xx, yy);
            spreadSpots = new ArrayList<Point>();
            spreadSpots.add(new Point(xx, yy));
            while (spreadSpots.size() > 0)
            {
                spreadWand((int)spreadSpots.get(0).getX(), (int)spreadSpots.get(0).getY(), 0);
                spreadSpots.remove(0);
            }
        }
    }
    public boolean compareColors(Color c1, Color c2)
    {
        return Math.abs(c1.getRed() - c2.getRed()) + Math.abs(c1.getGreen() - c2.getGreen()) + Math.abs(c1.getBlue() - c2.getBlue()) + Math.abs(c1.getAlpha() - c2.getAlpha()) < tolerance + 1;
    }
    public boolean spreadWand(int x, int y, int count)
    {
        selected[y][x] = true;
        boolean stopped = count > 5;
        if (x > 0 && !selected[y][x - 1] && compareColors(image.getColorAt(x - 1, y), wandPath))
            if (stopped)
                spreadSpots.add(new Point(x - 1, y));
            else
                stopped = spreadWand(x - 1, y, count + 1);
        if (x < image.getWidth() - 1 && !selected[y][x + 1] && compareColors(image.getColorAt(x + 1, y), wandPath))
            if (stopped)
                spreadSpots.add(new Point(x + 1, y));
            else
                stopped = spreadWand(x + 1, y, count + 1);
        if (y > 0 && !selected[y - 1][x] && compareColors(image.getColorAt(x, y - 1), wandPath))
            if (stopped)
                spreadSpots.add(new Point(x, y - 1));
            else
                stopped = spreadWand(x, y - 1, count + 1);
        if (y < image.getHeight() - 1 && !selected[y + 1][x] && compareColors(image.getColorAt(x, y + 1), wandPath))
            if (stopped)
                spreadSpots.add(new Point(x, y + 1));
            else
                stopped = spreadWand(x, y + 1, count + 1);
        return stopped;
    }
    public void deleteSelected()
    {
        if (selected == null)
            return;
        for (int y = 0; y < selected.length; y++)
            for (int x = 0; x < selected[0].length; x++)
                if (selected[y][x])
                    image.setColorAt(x, y, new Color(0, 0, 0, 0));
    }
    public Layer newLayerFromSelected(boolean cut)
    {
        if (selected == null)
            return null;
        GreenfootImage p = null;
        Point pos = null;
        Point last = null;
        for (int y = 0; y < selected.length; y++)
            for (int x = 0; x < selected[0].length; x++)
                if (selected[y][x])
                {
                    if (p == null)
                    {
                        p = new GreenfootImage(image.getWidth() - x, image.getHeight() - y);
                        pos = new Point(x, y);
                    }
                    if (x < pos.getX())
                    {
                        GreenfootImage pp = new GreenfootImage(image.getWidth() - x, p.getHeight());
                        pp.drawImage(p, (int)pos.getX() - x, 0);
                        p = pp;
                        pos = new Point(x, (int)pos.getY());
                    }
                    p.setColorAt(x - (int)pos.getX(), y - (int)pos.getY(), image.getColorAt(x, y));
                    if (last == null)
                        last = new Point(x, y);
                    else
                    {
                        if (x > last.getX())
                            last = new Point(x, (int)last.getY());
                        if (y > last.getY())
                            last = new Point((int)last.getX(), y);
                    }
                }
        if (p == null)
            return null;
        GreenfootImage pic = new GreenfootImage((int)(last.getX() - pos.getX()) + 1, (int)(last.getY() - pos.getY()) + 1);
        pic.drawImage(p, 0, 0);
        if (cut)
            deleteSelected();
        deselect();
        return new Layer(pic, new Point((int)(position.getX() + pos.getX()), (int)(position.getY() + pos.getY())));
    }
    public void fillAt(int x, int y, Color c)
    {
        Point p = new Point(x - (int)position.getX(), y - (int)position.getY());
        if (p.getX() < 0 || p.getX() > image.getWidth() - 1 || p.getY() < 0 || p.getY() > image.getHeight() - 1)
            return;
        fillPath = image.getColorAt((int)p.getX(), (int)p.getY());
        if (fillPath.equals(c))
            return;
        fillingColor = c;
        spreadSpots = new ArrayList<Point>();
        spreadSpots.add(new Point((int)p.getX(), (int)p.getY()));
        while (spreadSpots.size() > 0)
        {
            spreadFill((int)spreadSpots.get(0).getX(), (int)spreadSpots.get(0).getY(), 0);
            spreadSpots.remove(0);
        }
    }
    private Color fillPath, fillingColor;
    private ArrayList<Point> spreadSpots;
    public boolean spreadFill(int x, int y, int count)
    {
        image.setColorAt(x, y, fillingColor);
        boolean stopped = count > 5;
        if (x > 0 && image.getColorAt(x - 1, y).equals(fillPath))
            if (stopped)
                spreadSpots.add(new Point(x - 1, y));
            else
                stopped = spreadFill(x - 1, y, count + 1);
        if (x < image.getWidth() - 1 && image.getColorAt(x + 1, y).equals(fillPath))
            if (stopped)
                spreadSpots.add(new Point(x + 1, y));
            else
                stopped = spreadFill(x + 1, y, count + 1);
        if (y > 0 && image.getColorAt(x, y - 1).equals(fillPath))
            if (stopped)
                spreadSpots.add(new Point(x, y - 1));
            else
                stopped = spreadFill(x, y - 1, count + 1);
        if (y < image.getHeight() - 1 && image.getColorAt(x, y + 1).equals(fillPath))
            if (stopped)
                spreadSpots.add(new Point(x, y + 1));
            else
                stopped = spreadFill(x, y + 1, count + 1);
        return stopped;
    }
    public void flattenWith(ArrayList<Layer> layers)
    {
        Point oPos = new Point(position);
        Point mPos = new Point(getRightSide(), getBottomSide());
        for (Layer l : layers)
        {
            if (l.getPosition().getX() < oPos.getX())
                oPos = new Point((int)l.getPosition().getX(), (int)oPos.getY());
            if (l.getRightSide() > mPos.getX())
                mPos = new Point((int)l.getRightSide(), (int)mPos.getY());
            if (l.getPosition().getY() < oPos.getY())
                oPos = new Point((int)oPos.getX(), (int)l.getPosition().getY());
            if (l.getBottomSide() > mPos.getY())
                mPos = new Point((int)mPos.getX(), (int)l.getBottomSide());
        }
        GreenfootImage pic = new GreenfootImage((int)(mPos.getX() - oPos.getX()), (int)(mPos.getY() - oPos.getY()));
        for (int i = layers.size() - 1; i > -1; i--)
            pic.drawImage(layers.get(i).getImage(), (int)(layers.get(i).getPosition().getX() - oPos.getX()), (int)(layers.get(i).getPosition().getY() - oPos.getY()));
        pic.drawImage(image, (int)(position.getX() - oPos.getX()), (int)(position.getY() - oPos.getY()));
        position =  oPos;
        image = pic;
    }
    public void scale(int w, int h)
    {
        GreenfootImage destImage = new GreenfootImage(w, h); 
        Graphics2D g = destImage.getAwtImage().createGraphics(); 
        // Now enable interpolation. 
        g.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR); 
        g.drawImage(image.getAwtImage(), 0, 0, w, h, null);
        g.dispose();
        image = destImage;
    }
    public boolean fitBounds()
    {
        int y1 = -1;
        Outer:  for (int i = 0; i < image.getHeight(); i++)
                    for (int k = 0; k < image.getWidth(); k++)
                        if (image.getColorAt(k, i).getAlpha() != 0)
                        {
                            y1 = i;
                            break Outer;
                        }
        if (y1 == -1)
            return true;
        int y2 = y1;
        Outer:  for (int i = image.getHeight() - 1; i > y1 ; i--)
                    for (int k = 0; k < image.getWidth(); k++)
                        if (image.getColorAt(k, i).getAlpha() != 0)
                        {
                            y2 = i;
                            break Outer;
                        }
        int x1 = -1;
        Outer:  for (int k = 0; k < image.getWidth(); k++)
                    for (int i = y1; i < y2 + 1; i++)
                        if (image.getColorAt(k, i).getAlpha() != 0)
                        {
                            x1 = k;
                            break Outer;
                        }
        int x2 = x1;
        Outer:  for (int k = image.getWidth() - 1; k > x1; k--)
                    for (int i = y1; i < y2 + 1; i++)
                        if (image.getColorAt(k, i).getAlpha() != 0)
                        {
                            x2 = k;
                            break Outer;
                        }
        GreenfootImage pic = new GreenfootImage(x2 - x1 + 1, y2 - y1 + 1);
        pic.drawImage(image, -x1, -y1);
        image = pic;
        position.translate(x1, y1);
        return false;
    }
    public boolean eraseBetween(Point last, Point p, int size)
    {
        if (erase((int)p.getX(), (int)p.getY(), size))
            return true;
        double x = last.getX();
        double y = last.getY();
        double angle = Math.atan2(p.getY() - last.getY(), p.getX() - last.getX());
        while (true)
        {
            if (Math.abs(Math.atan2(p.getY() - y, p.getX() - x) - angle) > .01)
                break;
            if (erase((int)x, (int)y, size))
                return true;
            x += Math.cos(angle) * (size / 2d);
            y += Math.sin(angle) * (size / 2d);
        }
        return false;
    }
    public boolean erase(int x, int y, int size)
    {
        for (int i = 0; i < size; i++)
        {
            int ii = y - (int)position.getY() - size / 2 + i;
            if (ii > -1 && ii < image.getHeight())
                for (int k = 0; k < size; k++)
                {
                    int kk = x - (int)position.getX() - size / 2 + k;
                    if (kk > -1 && kk < image.getWidth())
                        image.setColorAt(kk, ii, new Color(0, 0, 0, 0));
                }
        }
        return fitBounds();
    }
    public boolean trimmingCanvas(Point size)
    {
        int x = getX();
        int y = getY();
        for (int i = 0; i < image.getHeight(); i++)
            for (int k = 0; k < image.getWidth(); k++)
                if (y + i < 0 || y + i > size.getY() || x + k < 0 || x + k > size.getX())
                    image.setColorAt(k, i, new Color(0, 0, 0, 0));
        return fitBounds();
    }
    public void rotateImageOneTurnClockWise()
    {
        GreenfootImage p = new GreenfootImage(image.getHeight(), image.getWidth());
        for (int i = 0; i < image.getHeight(); i++)
            for (int k = 0; k < image.getWidth(); k++)
                p.setColorAt(image.getHeight() - 1 - i, k, image.getColorAt(k, i));
        image = p;
    }
    public Color getColorAt(int x, int y)
    {
        x -= position.getX();
        y -= position.getY();
        if (x < 0 || x > image.getWidth() - 1 || y < 0 || y > image.getHeight() - 1)
            return null;
        return image.getColorAt(x, y);
    }
    public Point getPosition()
    {
        return position;
    }
    public int getX()
    {
        return (int)position.getX();
    }
    public int getY()
    {
        return (int)position.getY();
    }
    public void setPosition(Point p)
    {
        position = p;
    }
    public int getRightSide()
    {
        return getX() + image.getWidth();
    }
    public int getBottomSide()
    {
        return getY() + image.getHeight();
    }
    public int getWidth()
    {
        return image.getWidth();
    }
    public int getHeight()
    {
        return image.getHeight();
    }
}