import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import greenfoot.core.WorldHandler;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;
import java.awt.Color;
import java.awt.Point;

/**
 * A ListBox
 * 
 * @author Taylor Born
 * @version February 2011
 */
public class ListBox  extends Actor
{
    // Size of this ListBox
    private Point size;
    
    // Height of scrollBar
    private int scrollBar = 40;
    
    // Y position of where mouse pressed on scrollBar, used as reference for when dragged
    private int mouseY;
    
    private int textHeight = 14;
    private ArrayList<String> strs = new ArrayList<String>();
    private int scroll;
    private ArrayList<Integer> index = new ArrayList<Integer>();
    private boolean allowMultipleSelect = false;
    private boolean dragging;
    private Point lastMouse = new Point(-25, -25);
    
    private boolean changed = false;
    
    private ScrollingListener scroller = new ScrollingListener();
    private JPanel panel = WorldHandler.getInstance().getWorldCanvas();
    
    /**
     * Create a new ListBox.
     * @param size The width and height of the ListBox.
     * @param strings The list of Strings to display in the ListBox.
     */
    public ListBox(Point size, ArrayList<String> strings)
    {
        this.size = size;
        strs = strings;
        panel.addMouseWheelListener(scroller);
    }
    
    private void paint()
    {
        GreenfootImage pic = new GreenfootImage((int)size.getX(), (int)size.getY());
        pic.setColor(Color.WHITE);
        pic.fill();
        int scrollValue = (int)(((double)scroll / (size.getY() - scrollBar)) * (strs.size() * textHeight - size.getY() + 1));
        for (int i = 0; i < strs.size(); i++)
        {
            if (index.contains(i))
            {
                pic.setColor(Color.YELLOW);
                pic.fillRect(0, textHeight * i - scrollValue, (int)size.getX() - 10, textHeight);
            }
            else if (!dragging && lastMouse.getX() - (getX() - size.getX() / 2) > 0 && lastMouse.getX() - (getX() - size.getX() / 2) < size.getX() - 10 && lastMouse.getY() - (getY() - size.getY() / 2) > 0 && lastMouse.getY() - (getY() - size.getY() / 2) < size.getY() && lastMouse.getY() - (getY() - size.getY() / 2) > textHeight * i - scrollValue && lastMouse.getY() - (getY() - size.getY() / 2) <= textHeight * i - scrollValue + textHeight)
            {
                pic.setColor(new Color(185, 211, 225));
                pic.fillRect(0, textHeight * i - scrollValue, (int)size.getX() - 10, textHeight);
            }
            pic.setColor(Color.BLACK);
            pic.drawRect(0, textHeight * i - scrollValue, (int)size.getX() - 10, textHeight);
            pic.drawString(strs.get(i), 4, textHeight * i - scrollValue + textHeight - 2);
        }
        int n = scroller.getScroll();
        if (strs.size() > size.getY() / textHeight)
        {
            if (getWorld() != null && lastMouse.getX() > getX() - size.getX() / 2 && lastMouse.getX() < getX() + size.getX() / 2 && lastMouse.getY() > getY() - size.getY() / 2 && lastMouse.getY() < getY() + size.getY() / 2)
            {
                scroll += n;
                if (scroll < 0)
                    scroll = 0;
                else if (scroll + scrollBar > size.getY())
                    scroll = (int)size.getY() - scrollBar;
            }
            pic.setColor(Color.RED);
            pic.fillRect((int)size.getX() - 9, scroll, 9, scrollBar);
        }
        pic.setColor(Color.BLACK);
        pic.drawRect(0, 0, (int)size.getX() - 1, (int)size.getY() - 1);
        setImage(pic);
    }
    
    public void act()
    {
        MouseInfo mouse = Greenfoot.getMouseInfo();
        if (strs.size() > size.getY() / textHeight && Greenfoot.mousePressed(this) && mouse.getX() - (getX() - size.getX() / 2) > size.getX() - 10 && mouse.getX() - (getX() - size.getX() / 2) < size.getX() && mouse.getY() - (getY() - size.getY() / 2) > scroll && mouse.getY() - (getY() - size.getY() / 2) < scrollBar + scroll)
        {
            dragging = true;
            mouseY = (int)mouse.getY() - (getY() - (int)size.getY() / 2) - scroll;
        }
        if (Greenfoot.mouseClicked(null) || Greenfoot.mouseDragEnded(null))
            dragging = false;
        if (dragging && Greenfoot.mouseDragged(null))
        {
            scroll += mouse.getY() - (getY() - size.getY() / 2) - scroll - mouseY;
            if (scroll < 0)
                scroll = 0;
            else if (scroll + scrollBar > size.getY())
                scroll = (int)size.getY() - scrollBar;
        }
        if (Greenfoot.mouseDragged(null) || Greenfoot.mouseMoved(null))
            lastMouse = new Point(mouse.getX(), mouse.getY());
        if (Greenfoot.mousePressed(this) && mouse.getX() - (getX() - size.getX() / 2) > 0 && mouse.getX() - (getX() - size.getX() / 2) < size.getX() - 10 && mouse.getY() - (getY() - size.getY() / 2) > 0 && mouse.getY() - (getY() - size.getY() / 2) < size.getY())
        {
            for (int i = 0; i < strs.size(); i++)
                if (mouse.getY() - (getY() - size.getY() / 2) > textHeight * i - (int)(((double)scroll / (size.getY() - scrollBar)) * (strs.size() * textHeight - size.getY() + 1)) && mouse.getY() - (getY() - size.getY() / 2) <= textHeight * i - (int)(((double)scroll / (size.getY() - scrollBar)) * (strs.size() * textHeight - size.getY() + 1)) + textHeight)
                {
                    if (Greenfoot.isKeyDown("shift") && allowMultipleSelect)
                    {
                        if (index.contains(i))
                            index.remove(new Integer(i));
                        else
                            index.add(i);
                        changed = true;
                    }
                    else
                    {
                        if (index.size() != 1 || index.get(0) != i)
                            changed = true;
                        index = new ArrayList<Integer>();
                        index.add(i);
                    }
                }
        }
        scrollBar = (int)(((double)size.getY() / (strs.size() * textHeight + 1)) * size.getY());
        if (scrollBar < 5)
            scrollBar = 5;
        paint();
    }
    
    public void addedToWorld(World world)
    {
        scroll = 0;
        index = new ArrayList<Integer>();
        paint();
    }
    
    /**
     * The action listener for the ListBox.
     * @return Whether state of the selected elements has changed.
     */
    public boolean hasChanged()
    {
        boolean c = changed;
        changed = false;
        return c;
    }
    
    /**
     * Get the indexes of what is selected. Meant to be used when multiple selecting is enabled, otherwise getOneIndex() is more appropriate.
     * @return The indexes of what is selected.
     * @see getOneIndex()
     */
    public ArrayList<Integer> getIndexes()
    {
        return index;
    }
    
    /**
     * Get the index of what is selected. Meant to be used when multiple selecting is disabled, otherwise getIndexes() is more appropriate.
     * @return The index of what is selected. -1 if none selected.
     * @see getIndexes()
     */
    public int getOneIndex()
    {
        if (index.size() == 0)
            return -1;
        else
            return index.get(0);
    }
    
    
    /**
     * Set which item to be selected.
     * @param index The index of the item to be selected.
     */
    public void setIndex(int index)
    {
        if (index < -1 || index > strs.size() - 1)
            throw new IndexOutOfBoundsException();
        if (this.index.size() != 1 || this.index.get(0) != index)
            changed = true;
        this.index.clear();
        if (index != -1)
            this.index.add(index);
    }
    
    /**
     * Cause an element to be selected. Meant to be used when multiple selecting is enabled.
     * @param index The index of the item to be selected.
     */
    public void addIndex(int index)
    {
        if (index < 0 || index > strs.size() - 1)
            throw new IndexOutOfBoundsException();
        if (!this.index.contains(index))
        {
            this.index.add(index);
            changed = true;
        }
    }
    
    public void selectAll()
    {
        for (int i = 0; i < strs.size(); i++)
            addIndex(i);
    }
    
    /**
     * Determine if there are any elements selected.
     * @return Whether there are any elements selected.
     */
    public boolean hasSelection()
    {
        return index.size() > 0;
    }
    
    /**
     * Toggle the element at the given index to be selected or not be selected.
     * @param index The index of element to toggle selection.
     */
    public void toggleSelect(int index)
    {
        if (index < 0 || index > strs.size() - 1)
            throw new IndexOutOfBoundsException();
        if (this.index.contains(index))
            this.index.remove(new Integer(index));
        else
            this.index.add(index);
        changed = true;
    }
    
    /**
     * Swap two elements in position. If items are selected, they will remain selected after the swap.
     * @param indexA Index of element to swap position
     * @param indexB Index of element to swap position
     */
    public void swap(int indexA, int indexB)
    {
        if (indexA < 0 || indexA > strs.size() - 1 || indexB < 0 || indexB > strs.size() - 1)
            throw new IndexOutOfBoundsException();
        boolean a = index.contains(indexA);
        boolean b = index.contains(indexB);
        String s = strs.get(indexA);
        strs.set(indexA, strs.get(indexB));
        strs.set(indexB, s);
        index.remove(new Integer(indexA));
        index.remove(new Integer(indexB));
        if (a)
            index.add(indexB);
        if (b)
            index.add(indexA);
    }
    
    /**
     * Get the selected String. Meant to be used when multiple selecting is disabled, otherwise getSelection() is more appropriate.
     * @return The selected String. Null if none selected.
     * @see getSelection()
     */
    public String getOneSelection()
    {
        if (index.size() == 0)
            return null;
        return strs.get(index.get(0));
    }
    
    /**
     * Get all selected Strings. Meant to be used when multiple selecting is enabled, otherwise getOneSelection() is more appropriate.
     * @return The list of selected Strings.
     * @see getOneSelection()
     */
    public ArrayList<String> getSelection()
    {
        ArrayList<String> s = new ArrayList<String>();
        for (int i = 0; i < strs.size(); i++)
            if (index.contains(i))
                s.add(strs.get(i));
        return s;
    }
    
    /**
     * Get the String in the ListBox at the given index.
     * @param index The index for what String in the ListBox to get.
     * @return The String at the index of the ListBox.
     */
    public String getString(int index)
    {
        if (index < 0 || index > strs.size() - 1)
            throw new IndexOutOfBoundsException();
        return strs.get(index);
    }
    
    /**
     * Get the contents of the ListBox.
     * @return The contents of the ListBox.
     */
    public ArrayList<String> getList()
    {
        return strs;
    }
    
    /**
     * Add a String to the ListBox, making it selected and in view.
     * @param s The String to be added.
     */
    public void addString(String s)
    {
        strs.add(s);
        index = new ArrayList<Integer>();
        index.add(strs.size() - 1);
        scrollBar = (int)(((double)size.getY() / (strs.size() * textHeight + 1)) * size.getY());
        if (scrollBar < 5)
            scrollBar = 5;
        if (strs.size() * textHeight - 2 > size.getY())
            scroll = (int)size.getY() - scrollBar;
        changed = true;
    }
    
    /**
     * Insert a String to the ListBox at the given position, making it selected and in view.
     * @param s The String to be inserted.
     * @param index The position to insert String.
     */
    public void addStringAt(String s, int index)
    {
        if (index < 0 || index > strs.size())
            throw new IndexOutOfBoundsException();
        strs.add(index, s);
        this.index = new ArrayList<Integer>();
        this.index.add(index);
        //
        scrollBar = (int)(((double)size.getY() / (strs.size() * textHeight + 1)) * size.getY());
        if (scrollBar < 5)
            scrollBar = 5;
        if (strs.size() * textHeight - 2 > size.getY())
            scroll = (int)size.getY() - scrollBar;
        changed = true;
    }
    
    /**
     * Remove what is selected.
     */
    public void removeSelected()
    {
        if (index.size() != 0)
            changed = true;
        ArrayList<String> strsD = new ArrayList<String>();
        for (Integer i : index)
            strsD.add(strs.get(i));
        for (String s : strsD)
            strs.remove(s);
        index = new ArrayList<Integer>();
        if (strs.size() * textHeight - 2 < size.getY())
            scroll = 0;
    }
    
    public void remove(int i)
    {
        strs.remove(i);
        if (index.contains(new Integer(i)))
        {
            index.remove(new Integer(i));
            changed = true;
        }
    }
    
    /**
     * Get the number of items in the ListBox.
     * @return Number of items in the ListBox.
     */
    public int getSize()
    {
        return strs.size();
    }
    
    /**
     * Remove everything from the ListBox.
     */
    public void clear()
    {
        strs.clear();
        changed = index.size() != 0;
        index.clear();
    }
    
    /**
     * Change the String at a given index.
     * @param index The index in the ListBox to change.
     * @param string The String to set to at the given index.
     */
    public void set(int index, String string)
    {
        if (index < 0 || index > strs.size() - 1)
            throw new IndexOutOfBoundsException();
        strs.set(index, string);
        changed = true;
    }
    
    /**
     * Determine if the ListBox is set to allow multiple selecting of items.
     * @return If the ListBox is set to allow multiple selecting of items.
     */
    public boolean multipleSelectingIsEnabled()
    {
        return allowMultipleSelect;
    }
    
    /**
     * Change if the ListBox is set to allow multiple selecting of items.
     * @param enable If the ListBox will be set to allow multiple selecting of items.
     */
    public void setMultipleSelecting(boolean enable)
    {
        allowMultipleSelect = enable;
    }
    
    class ScrollingListener implements MouseWheelListener
    {
        int amount = 0;
        
        public void mouseWheelMoved(MouseWheelEvent e)
        {
            amount += e.getWheelRotation();
            e.consume();
        }
        
        public int getScroll()
        {
            int s = amount;
            amount = 0;
            return s;
        }
    }
}