import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.*;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Color;
import java.awt.Point;

/**
 * A Menu
 * 
 * @author Taylor Born
 * @version March 2011
 */
public class Menu  extends Actor
{
    static Font font = new Font("New Times Roman", Font.PLAIN, 12);
    private static int checkMarkWidth = 16;

    private Point home;
    private Point lastMouse = new Point(-25, -25);
    private ArrayList<MenuItem> items = new ArrayList<MenuItem>();
    private ArrayList<MenuSet> sets = new ArrayList<MenuSet>();
    private String active = "";
    private String itemPressed = null;
    private Point origin;
    
    private String keysPressed = "";
    
//     static
//     {
//         Graphics2D graphics = (new GreenfootImage(1, 1)).getAwtImage().createGraphics();
//         graphics.setFont(font);
//         FontMetrics fm = graphics.getFontMetrics();
//         checkMarkWidth = fm.charsWidth("✓ ".toCharArray(), 0, "✓ ".length());
//     }
    
    /**
     * Create the Menu with the initial Menu "headings".
     * @param menuItems This first list are the Menu items that are always displayed on the "top" 
     * bar. Menu items can be added to this "top" bar with the adding methods, just don't need 
     * parent Menu items when doing so.
     */
    public Menu(ArrayList<String> menuItems)
    {
        for (String s : menuItems)
            items.add(new MenuItem(s));
    }
    
    /**
     * Create the Menu with the initial Menu "headings", with a top left desired origin position.
     * @param menuItems This first list are the Menu items that are always displayed on the "top" 
     * bar. Menu items can be added to this "top" bar with the adding methods, just don't need 
     * parent Menu items when doing so.
     * @param p Top left desired origin position. new Point(0, 0) will make sure the Menu is snug in the top left corner of the World
     */
    public Menu(ArrayList<String> menuItems, Point p)
    {
        this(menuItems);
        origin = p;
    }
    
    public void act() 
    {
        MouseInfo mouse = Greenfoot.getMouseInfo();
        if (Greenfoot.mouseMoved(null) || Greenfoot.mouseDragged(null))
            lastMouse = new Point(mouse.getX(), mouse.getY());
        
        int height = 12;
        
        int width = 0;
        
        int barWidth = 0;
        for (MenuItem mi : items)
            barWidth += mi.getWidth();
        
        if (active.length() == 0)
            width = barWidth;
        else
        {
            for (int i = 0; i < items.size(); i++)
                if (items.get(i).getName().equals(active.substring(0, active.indexOf("/"))))
                {
                    width += items.get(i).getRunningWidth(active.substring(active.indexOf("/") + 1), true);
                    height += 12 + items.get(i).getRunningHeight(active.substring(active.indexOf("/") + 1));
                    break;
                }
                else
                    width += items.get(i).getWidth();
            if (barWidth > width)
                width = barWidth;
        }
        
        setLocation((int)home.getX() + (width - barWidth) / 2 - (width % 2 == 0 ? 0 : 1), (int)home.getY() + (height - 12) / 2 - ((height + 1) % 2 != 0 ? 1 : 0));
        Point offsetMouse = new Point((int)lastMouse.getX() - (getX() - (width / 2)), (int)lastMouse.getY() - (getY() - (height / 2)));
        
        boolean clicked = Greenfoot.mouseClicked(null);
        if (Greenfoot.mouseClicked(this) && offsetMouse.getX() > 0 && offsetMouse.getX() < width && offsetMouse.getY() > 0 && offsetMouse.getY() < 12)
        {
            int runX = 0;
            for (int i = 0; i < items.size(); i++)
                if (runX + items.get(i).getWidth() > offsetMouse.getX())
                {
                    if (active.startsWith(items.get(i).getName() + "/"))
                        active = "";
                    else
                        active = items.get(i).getName() + "/";
                    break;
                }
                else
                    runX += items.get(i).getWidth();
            clicked = false;
        }
        
        GreenfootImage pic = new GreenfootImage(width + (width % 2 != 0 ? 1 : 2), height + 1);
        pic.setFont(font);
        
        int runWidth = 0;
        for (MenuItem barItem : items)
        {
            pic.setColor(Color.BLACK);
            if (active.length() != 0 && offsetMouse.getX() > runWidth && offsetMouse.getX() <= runWidth + barItem.getWidth() && offsetMouse.getY() > 0 && offsetMouse.getY() <= 12)
            {
                active = barItem.getName() + "/";
                if (clicked)
                    itemPressed = active;
            }
            if (active.startsWith(barItem.getName() + "/"))
            {
                String s = active.substring(active.indexOf("/") + 1);
                int runningX = runWidth;
                int runningY = 12;
                MenuItem item = barItem;
                boolean sneakpeek = s.length() == 0 && item.isEnabled() && item.getItems().size() > 0;
                String address = item.getName() + "/";
                while (s.length() != 0 || sneakpeek)
                {
                    int indexOfNext = -1;
                    for (int i = 0; i < item.getItems().size(); i++)
                    {
                        pic.setColor(Color.WHITE);
                        if (item.getItems().get(i).isEnabled())
                            if (!sneakpeek && item.getItems().get(i).getName().equals(s.substring(0, s.indexOf("/"))))
                            {
                                indexOfNext = i;
                                pic.setColor(Color.CYAN);
                            }
                        int w = item.getLargestChild();
                        if (offsetMouse.getX() > runningX && offsetMouse.getX() <= runningX + w && offsetMouse.getY() > runningY + i * 12 && offsetMouse.getY() <= runningY + i * 12 + 12)
                        {
                            active = address + item.getItems().get(i).getName() + "/";
                            if (item.getItems().get(i).isEnabled())
                            {
                                if (clicked)
                                {
                                    itemPressed = active;
                                    checkSets(item.getItems().get(i));
                                }
                                pic.setColor(Color.CYAN);
                            }
                            else
                                pic.setColor(Color.WHITE);
                        }
                        pic.fillRect(runningX, runningY + i * 12, w, 12);
                        if (item.getItems().get(i).getItems().size() > 0)
                        {
                            if (item.getItems().get(i).isEnabled())
                                pic.setColor(Color.GRAY);
                            else
                                pic.setColor(new Color(220, 220, 220));
                            int[] xs = new int[3];
                            xs[0] = runningX + w - 8;
                            xs[1] = runningX + w - 1;
                            xs[2] = runningX + w - 8;
                            int[] ys = new int[3];
                            ys[0] = runningY + i * 12 + 2;
                            ys[1] = runningY + i * 12 + 6;
                            ys[2] = runningY + i * 12 + 10;
                            pic.fillPolygon(xs, ys, 3);
                        }
                        if (item.getItems().get(i).isChecked() && runningY + i * 12 + 2 + 8 < pic.getHeight())
                        {
                            pic.setColorAt(runningX + 2 + 6, runningY + i * 12 + 2 + 0, new Color(121, 121, 121));
                            pic.setColorAt(runningX + 2 + 7, runningY + i * 12 + 2 + 0, new Color(0, 0, 0));
                            pic.setColorAt(runningX + 2 + 8, runningY + i * 12 + 2 + 0, new Color(194, 194, 194));
                            pic.setColorAt(runningX + 2 + 5, runningY + i * 12 + 2 + 1, new Color(138, 138, 138));
                            pic.setColorAt(runningX + 2 + 6, runningY + i * 12 + 2 + 1, new Color(0, 0, 0));
                            pic.setColorAt(runningX + 2 + 7, runningY + i * 12 + 2 + 1, new Color(189, 189, 189));
                            pic.setColorAt(runningX + 2 + 5, runningY + i * 12 + 2 + 2, new Color(0, 0, 0));
                            pic.setColorAt(runningX + 2 + 6, runningY + i * 12 + 2 + 2, new Color(137, 137, 137));
                            pic.setColorAt(runningX + 2 + 4, runningY + i * 12 + 2 + 3, new Color(138, 138, 138));
                            pic.setColorAt(runningX + 2 + 5, runningY + i * 12 + 2 + 3, new Color(27, 27, 27));
                            pic.setColorAt(runningX + 2 + 1, runningY + i * 12 + 2 + 4, new Color(43, 43, 43));
                            pic.setColorAt(runningX + 2 + 2, runningY + i * 12 + 2 + 4, new Color(168, 168, 168));
                            pic.setColorAt(runningX + 2 + 4, runningY + i * 12 + 2 + 4, new Color(28, 28, 28));
                            pic.setColorAt(runningX + 2 + 5, runningY + i * 12 + 2 + 4, new Color(151, 151, 151));
                            pic.setColorAt(runningX + 2 + 1, runningY + i * 12 + 2 + 5, new Color(122, 122, 122));
                            pic.setColorAt(runningX + 2 + 2, runningY + i * 12 + 2 + 5, new Color(0, 0, 0));
                            pic.setColorAt(runningX + 2 + 3, runningY + i * 12 + 2 + 5, new Color(93, 93, 93));
                            pic.setColorAt(runningX + 2 + 4, runningY + i * 12 + 2 + 5, new Color(16, 16, 16));
                            pic.setColorAt(runningX + 2 + 2, runningY + i * 12 + 2 + 6, new Color(42, 42, 42));
                            pic.setColorAt(runningX + 2 + 3, runningY + i * 12 + 2 + 6, new Color(0, 0, 0));
                            pic.setColorAt(runningX + 2 + 4, runningY + i * 12 + 2 + 6, new Color(93, 93, 93));
                            pic.setColorAt(runningX + 2 + 2, runningY + i * 12 + 2 + 7, new Color(190, 190, 190));
                            pic.setColorAt(runningX + 2 + 3, runningY + i * 12 + 2 + 7, new Color(0, 0, 0));
                            pic.setColorAt(runningX + 2 + 4, runningY + i * 12 + 2 + 7, new Color(190, 190, 190));
                            pic.setColorAt(runningX + 2 + 3, runningY + i * 12 + 2 + 8, new Color(131, 131, 131));
                        }
                        pic.setColor(Color.BLACK);
                        pic.drawRect(runningX, runningY + i * 12, w, 12);
                        if (!item.getItems().get(i).isEnabled())
                            pic.setColor(Color.GRAY);
                        pic.drawString(item.getItems().get(i).getName(), runningX + 2 + (item.getItems().get(i).isMemberOfSet() ? checkMarkWidth : 0), runningY + i * 12 + 11);
                        if (item.getItems().get(i).getShortCut() != null)
                            pic.drawString(item.getItems().get(i).getShortCut(), runningX + w - item.getItems().get(i).getWidthOfShortCut() - 2, runningY + i * 12 + 11);
                        //pic.drawString((item.getItems().get(i).isChecked() ? "✓ " : "") + item.getItems().get(i).getName(), runningX + 2 + (item.getItems().get(i).isMemberOfSet() && !item.getItems().get(i).isChecked() ? checkMarkWidth : 0), runningY + i * 12 + 11);
                    }
                    if (!sneakpeek)
                    {
                        s = s.substring(s.indexOf("/") + 1);
                        if (indexOfNext != -1)
                        {
                            runningX += item.getLargestChild();
                            runningY += indexOfNext * 12;
                            item = item.getItems().get(indexOfNext);
                            address += item.getName() + "/";
                            if (s.length() == 0 && item.getItems().size() > 0)
                                sneakpeek = true;
                        }
                    }
                    else
                    {
                        s = "";
                        sneakpeek = false;
                    }
                }
                pic.setColor(Color.CYAN);
            }
            else
                pic.setColor(Color.WHITE);
            pic.fillRect(runWidth, 0, barItem.getWidth(), 12);
            pic.setColor(Color.BLACK);
            pic.drawRect(runWidth, 0, barItem.getWidth(), 12);
            pic.drawString(barItem.getName(), runWidth + 2, 11);
            runWidth += barItem.getWidth();
        }
        if (clicked)
            active = "";
        
        scMI = null;
        scCount = 0;
        for (MenuItem mi : items)
            checkForShortCuts(mi, "");
        if (scMI != null)
        {
            keysPressed += " " + scMI.getShortCut() + " ";
//             for (String s = scMI.getShortCut().replace("ctrl", "control"); true; s = s.substring(s.indexOf("+") + 1))
//             {
//                 keysPressed += " " + s.substring(0, s.indexOf("+") != -1 ? s.indexOf("+") : s.length()) + " ";
//                 if (s.indexOf("+") == -1)
//                     break;
//             }
            checkSets(scMI);
        }
//         for (String s = keysPressed; s.length() > 1; s = s.substring(s.indexOf(" ") + 1))
//         {
//             if (s.startsWith(" "))
//                 s = s.substring(1);
//             if (!Greenfoot.isKeyDown(s.substring(0, s.indexOf(" "))))
//                 keysPressed = keysPressed.replace(" " + s.substring(0, s.indexOf(" ")) + " ", "");
//         }
        setImage(pic);
    }
    
    private void checkSets(MenuItem mi)
    {
        for (MenuSet set : sets)
            set.check(mi);
    }
    
    private MenuItem scMI;
    private int scCount;
    private void checkForShortCuts(MenuItem mi, String s)
    {
//         if (!mi.isEnabled())
//             return;
        if (mi.checkShortCut())
        {
            if (!keysPressed.contains(" " + mi.getShortCut() + " "))
            {
                boolean used = false;
                int c = 0;
                for (String ss = mi.getShortCut().replace("ctrl", "control"); true; ss = ss.substring(ss.indexOf("+") + 1))
                {
                    c++;
                    if (ss.indexOf("+") == -1)
                        break;
                }
    //             for (String ss = mi.getShortCut().replace("ctrl", "control"); true; ss = ss.substring(ss.indexOf("+") + 1))
    //             {
    //                 if (keysPressed.contains(" " + ss.substring(0, ss.indexOf("+") != -1 ? ss.indexOf("+") : ss.length()) + " "))
    //                     used = true;
    //                 c++;
    //                 if (ss.indexOf("+") == -1)
    //                     break;
    //             }
                if (!used && c > scCount)
                {
                    scCount = c;
                    scMI = mi;
                    itemPressed = s + mi.getName() + "/";
                }
            }
            
//             if (!keysPressed.contains(" " + mi.getShortCut() + " "))
//             {
//                 itemPressed = s + mi.getName() + "/";
//                 checkSets(mi);
//                 keysPressed += " " + mi.getShortCut() + " ";
//             }
        }
        else
            keysPressed = keysPressed.replace(" " + mi.getShortCut() + " ", "");
        for (MenuItem m : mi.getItems())
            checkForShortCuts(m, s + mi.getName() + "/");
    }
    
    /**
     * The action listener for the Menu.
     * @return String representation of what Menu item was last clicked in the format of the
     * path getting to it. For example if an "Arrow" Menu item was clicked: "Insert/Shape/Arrow/"
     */
    public String getItemPressed()
    {
        String s = itemPressed;
        itemPressed = null;
        return s;
    }
    
    /**
     * Add an additional Menu item to the Menu.
     * @param address Specify the path to the location to add something, with the item being 
     * added listed at the end. For example, to add "Square" in the list at "Insert/Shape/", 
     * input this: "Insert/Shape/Square/". Note that the parent Menu items must exist.
     * @see #addItems(String, ArrayList)
     */
    public void addItem(String address)
    {
        if (!address.contains("/"))
            items.add(new MenuItem(address));
        else
        {
            for (MenuItem mi : items)
                if (mi.getName().equals(address.substring(0, address.indexOf("/"))))
                {
                    mi.add(address.substring(address.indexOf("/") + 1));
                    break;
                }
        }
    }
    
    /**
     * Add an additional Menu item to the Menu that may be marked with and without a check mark. When clicking
     * on this item, it will be marked if not marked, or unmarked if already marked.
     * @param address Specify the path to the location to add something, with the item being 
     * added listed at the end. For example, to add "Square" in the list at "Insert/Shape/", 
     * input this: "Insert/Shape/Square/". Note that the parent Menu items must exist.
     * @param checked If the item will be initially marked with a check mark.
     */
    public void addToggleItem(String address, boolean checked)
    {
        String a = address;
        while (a.contains("/"))
            a = a.substring(address.indexOf("/") + 1);
        String s = address.substring(0, address.length() - a.length());
        ArrayList<String> arr = new ArrayList<String>();//{{ add(a); }};
        arr.add(a);
        addItemsAsSet(s, arr, checked ? a : null);
    }
    
    /**
     * Add multiple additional Menu items to the Menu into a single address.
     * @param address The address for where to add the Menu items. For example, "Insert/Shape/" 
     * will specify to add the Menu items under "Shape". Note that these parent Menu items must exist.
     * @param menuItems The list of Menu items to add to the Menu. For example, "Circle/" will add
     * a Menu item with the name "Circle" at the location specified by the address param.
     * @see #addItem(String)
     */
    public void addItems(String address, ArrayList<String> menuItems)
    {
        for (String str : menuItems)
            addItem(address + str);
    }
    
    /**
     * In addition to what addItems(String, ArrayList<String>) method does, the items added will belong to a set where one of them may be marked with a check mark.
     * When one item is clicked, it will be marked with a check mark while the others will be unmarked.
     * @param address The address for where to add the Menu items. For example, "Insert/Shape/" 
     * will specify to add the Menu items under "Shape". Note that these parent Menu items must exist.
     * @param items The list of Menu items to add to the Menu. For example, "Circle/" will add
     * a Menu item with the name "Circle" at the location specified by the address param.
     * @param itemSelected The item that will be marked with a check mark initially. May be null.
     * @see #addItems(String, ArrayList)
     * @see #addToggleItem(String, boolean)
     */
    public void addItemsAsSet(String address, ArrayList<String> items, String itemSelected)
    {
        addItems(address, items);
        if (address == null || address.length() == 0 || address.charAt(address.length() - 1) != '/' || items == null || items.size() == 0)
            return;
        MenuItem m = null;
        for (MenuItem mi : this.items)
            if (mi.getName().equals(address.substring(0, address.indexOf("/"))))
            {
                m = mi;
                address = address.substring(address.indexOf("/") + 1);
                break;
            }
        if (m == null)
            return;
        while (address.length() > 0)
        {
            boolean found = false;
            for (MenuItem mi : m.getItems())
            {
                if (mi.getName().equals(address.substring(0, address.indexOf("/"))))
                {
                    m = mi;
                    address = address.substring(address.indexOf("/") + 1);
                    found = true;
                    break;
                }
            }
            if (!found)
                return;
        }
        ArrayList<MenuItem> setItems = new ArrayList<MenuItem>();
        MenuItem st = null;
        for (MenuItem mi : m.getItems())
        {
            if (items.contains(mi.getName()))
            {
                setItems.add(mi);
                mi.addedToSet();
            }
            if (mi.getName().equals(itemSelected))
            {
                st = mi;
                st.toggleCheck();
            }
        }
        sets.add(new MenuSet(setItems, st));
    }
    
    /**
     * Create a shortcut to a specific item. This makes it not necessary to navigate through the Menu.
     * @param address The location of item. For example "File/Print".
     * @param shortCut Key combination that will provoke the call as though the item was clicked. Examples: "ctrl+p", "ctrl+shift+up", "tab+5".
     */
    public void addShortCut(String address, String shortCut)
    {
        MenuItem m = null;
        for (MenuItem mi : items)
        {
            if (mi.getName().equals(address.substring(0, address.indexOf("/"))))
                m = mi;
        }
        address = address.substring(address.indexOf("/") + 1);
        while (address.length() > 0)
        {
            for (MenuItem mi : m.getItems())
            {
                if (!address.contains("/"))
                {
                    if (mi.getName().equals(address))
                    {
                        mi.setShortCut(shortCut);
                        return;
                    }
                }
                else if (mi.getName().equals(address.substring(0, address.indexOf("/"))))
                {
                    m = mi;
                    address = address.substring(address.indexOf("/") + 1);
                    break;
                }
            }
        }
    }
    
    /**
     * Cause as though the specific item was clicked on. This is handy for when there are icons
     * elsewhere that the user may press that should do the same thing and you would wish check
     * marks to change in a set that the item belongs to. Note getItemPressed() will return as
     * though this item was clicked.
     * @param address The location of item. For example "File/Print".
     * @see getItemPressed()
     */
    public void provokeItem(String address)
    {
        itemPressed = address + "/";
        MenuItem m = null;
        for (MenuItem mi : items)
        {
            if (mi.getName().equals(address.substring(0, address.indexOf("/"))))
                m = mi;
        }
        address = address.substring(address.indexOf("/") + 1);
        while (address.length() > 0)
        {
            for (MenuItem mi : m.getItems())
            {
                if (!address.contains("/"))
                {
                    if (mi.getName().equals(address))
                    {
                        checkSets(mi);
                        return;
                    }
                }
                else if (mi.getName().equals(address.substring(0, address.indexOf("/"))))
                {
                    m = mi;
                    address = address.substring(address.indexOf("/") + 1);
                    break;
                }
            }
        }
    }
    
    /**
     * Change the enable state of an item.
     * @param address The location of item.
     * @param enable Whether the item will be enabled.
     */
    public void enableItem(String address, boolean enable)
    {
        MenuItem m = null;
        for (MenuItem mi : items)
        {
            if (mi.getName().equals(address.substring(0, address.indexOf("/"))))
                m = mi;
        }
        address = address.substring(address.indexOf("/") + 1);
        while (address.length() > 0)
        {
            for (MenuItem mi : m.getItems())
            {
                if (!address.contains("/"))
                {
                    if (mi.getName().equals(address))
                    {
                        mi.enable(enable);
                        return;
                    }
                }
                else if (mi.getName().equals(address.substring(0, address.indexOf("/"))))
                {
                    m = mi;
                    address = address.substring(address.indexOf("/") + 1);
                    break;
                }
            }
        }
    }
    
    /**
     * Whether the menu is being navigated
     * @return Whether the menu is being navigated
     */
    public boolean isActive()
    {
        return active.length() != 0;
    }
    
    public void addedToWorld(World world)
    {
        if (origin != null)
        {
            int barWidth = 0;
            for (MenuItem mi : items)
                barWidth += mi.getWidth();
            home = new Point((int)origin.getX() + barWidth / 2, (int)origin.getY() + 12 / 2);
        }
        else
            home = new Point(getX(), getY());
    }

    class MenuItem
    {
        private ArrayList<MenuItem> items = new ArrayList<MenuItem>();
        private String name;
        private boolean checked;
        private boolean memberOfSet;
        private String shortCut;
        private boolean enabled = true;
        
        public MenuItem(String n)
        {
            name = n;
        }
        
        public void add(String s)
        {
            if (!s.contains("/"))
                items.add(new MenuItem(s));
            else
            {
                for (MenuItem mi : items)
                    if (mi.getName().equals(s.substring(0, s.indexOf("/"))))
                    {
                        mi.add(s.substring(s.indexOf("/") + 1));
                        break;
                    }
            }
        }
        public int getWidth()
        {
            return getWidthOf(name) + 4;
//             Graphics2D graphics = (new GreenfootImage(1, 1)).getAwtImage().createGraphics();
//             graphics.setFont(font);
//             FontMetrics fm = graphics.getFontMetrics();
//             return fm.charsWidth(name.toCharArray(), 0, name.length()) + 4;
        }
        public int getWidthOf(String s)
        {
            Graphics2D graphics = (new GreenfootImage(1, 1)).getAwtImage().createGraphics();
            graphics.setFont(font);
            FontMetrics fm = graphics.getFontMetrics();
            return fm.charsWidth(s.toCharArray(), 0, s.length());
        }
        
        public int getRunningWidth(String s, boolean first)
        {
            int w = getLargestChild();
            if (s.length() != 0)
                for (MenuItem mi : items)
                    if (mi.getName().equals(s.substring(0, s.indexOf("/"))))
                        return w + mi.getRunningWidth(s.substring(s.indexOf("/") + 1), false);
            return w;
        }
        public int getLargestChild()
        {
            int n = 0;
            for (MenuItem mi : items)
            {
                int var = mi.getWidth() + (mi.getItems().size() > 0 ? 10 : 0) + (mi.isMemberOfSet() ? checkMarkWidth : 0) + (mi.getShortCut() != null ? getWidthOf("   " + mi.getShortCut()) : 0);
                if (var > n)
                    n = var;
            }
            return n;
        }
        
        public int getRunningHeight(String s)
        {
            if (items.size() == 0)
                return 0;
            int h = (items.size() - 1) * 12;
            if (s.length() != 0)
                for (MenuItem mi : items)
                    if (mi.getName().equals(s.substring(0, s.indexOf("/"))))
                        return h + mi.getRunningHeight(s.substring(s.indexOf("/") + 1));
            return h;
        }
        public ArrayList<MenuItem> getItems()
        {
            return items;
        }
        public String getName()
        {
            return name;
        }
        public void setShortCut(String s)
        {
            shortCut = s;
        }
        public String getShortCut()
        {
            return shortCut;
        }
        public int getWidthOfShortCut()
        {
            return getWidthOf(shortCut);
        }
        public boolean checkShortCut()
        {
            if (shortCut == null || !enabled)
                return false;
            for (String s = shortCut; s.length() > 0; s = (s.indexOf("+") == -1 ? "" : s.substring(s.indexOf("+") + 1)))
            {
                String ss = (s.indexOf("+") == -1 ? s : s.substring(0, s.indexOf("+")));
                if (ss.equals("ctrl"))
                {
                    if (!Greenfoot.isKeyDown("control"))
                        return false;
                }
                else if (!Greenfoot.isKeyDown(ss))
                    return false;
            }
            return true;
        }
        public boolean isChecked()
        {
            return checked;
        }
        public void toggleCheck()
        {
            checked = !checked;
        }
        public void addedToSet()
        {
            memberOfSet = true;
        }
        public boolean isMemberOfSet()
        {
            return memberOfSet;
        }
        public boolean isEnabled()
        {
            return enabled;
        }
        public void enable(boolean e)
        {
            enabled = e;
        }
    }
    class MenuSet
    {
        private ArrayList<MenuItem> items = new ArrayList<MenuItem>();
        private MenuItem checkItem;
        
        public MenuSet(ArrayList<MenuItem> items, MenuItem m)
        {
            this.items = items;
            checkItem = m;
        }
        public void check(MenuItem mi)
        {
            if (!items.contains(mi))
                return;
            if (items.size() == 1)
                mi.toggleCheck();
            else if (!mi.isChecked())
            {
                if (checkItem != null)
                    checkItem.toggleCheck();
                mi.toggleCheck();
                checkItem = mi;
            }
        }
    }
}