import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import greenfoot.core.WorldHandler;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;
import java.awt.Point;
import java.awt.Color;

/**
 * Write a description of class Screen here.
 * 
 * @author Taylor Born
 * @version November 2011
 */
public class Screen extends World
{
    static Screen main;
    private ArrayList<Layer> layers = new ArrayList<Layer>();
    private ListBox listBox = new ListBox(new Point(130, 100), new ArrayList<String>());
    private ArrayList<Layer> clipBoard = new ArrayList<Layer>();
    
    private Point scroll = new Point(0, 0);
    private int zoom = 1;
    private Point size = new Point(300, 250);
    
    private Color drawingColor = Color.BLACK;
    private Palette myPalette = new Palette();
    private Point lastPressedAt;
    private Slider brushSlider = new Slider(100, 1, 20, 2, 1);
    private Slider toleranceSlider = new Slider(100, 0, 255, 0, 1);
    private PropertyPanel propertyPanel = new PropertyPanel();
    
    private Menu menu = new Menu(new ArrayList<String>(){{ add("File"); add("Edit"); add("View"); add("Modify"); add("Tools"); }}, new Point(0, 0));
    private SizeInputPanel sizeInputPanel = new SizeInputPanel();
    private FileHandler fileHandler = new FileHandler();
    
    private Point lastMouse = new Point(-25, -25);
    
    private int count = 0;
    private int soCount = 0;
    
    private GreenfootImage toolMovePic = new GreenfootImage("ToolMove.png");
    private GreenfootImage toolDrawPic = new GreenfootImage("ToolDraw.png");
    private GreenfootImage toolFillPic = new GreenfootImage("ToolFill.png");
    private GreenfootImage toolWandPic = new GreenfootImage("ToolWand.png");
    private GreenfootImage toolErasePic = new GreenfootImage("ToolErase.png");
    private GreenfootImage toolDropperPic = new GreenfootImage("ToolDropper.png");
    
    public enum Tool { HANDLE, DRAW, FILL, WAND, ERASE, DROPPER }
    public Tool tool = Tool.DRAW;
    
    static boolean start = true;
    
    private static final int CODE_ID = 59;
    private boolean shownCode = false;
    
    public Screen()
    {
        super(600, 400, 1);
        main = this;
        CodeKeyUser.enter();
        panel.addMouseWheelListener(scroller);
        setPaintOrder(Palette.Cover.class, Menu.class); // , Slider.class, Button.class, Palette.class, ListBox.class
        listBox.setMultipleSelecting(true);
        addObject(listBox, 523, 331);
        addObject(brushSlider, 523, 100);
        addObject(toleranceSlider, 523, 100);
        toleranceSlider.hide();
        menu.addItems("File/", new ArrayList<String>(){{ add("New"); add("Open"); add("Save"); }} );
        menu.addShortCut("File/New", "ctrl+n");
        menu.addShortCut("File/Open", "ctrl+o");
        menu.addShortCut("File/Save", "ctrl+s");
        menu.addItems("Edit/", new ArrayList<String>(){{ add("Cut"); add("Copy"); add("Paste"); add("Clear"); add("Select All"); }});
        menu.addShortCut("Edit/Cut", "ctrl+x");
        menu.addShortCut("Edit/Copy", "ctrl+c");
        menu.addShortCut("Edit/Paste", "ctrl+v");
        menu.addShortCut("Edit/Clear", "backspace");
        menu.addShortCut("Edit/Select All", "ctrl+a");
        menu.addItems("View/", new ArrayList<String>(){{ add("Zoom In"); add("Zoom Out"); add("Magnification"); }});
        menu.addShortCut("View/Zoom In", "ctrl+=");
        menu.addShortCut("View/Zoom Out", "ctrl+-");
        menu.addItemsAsSet("View/Magnification/", new ArrayList<String>(){{ add("100%"); add("300%"); add("500%"); add("700%"); add("1000%"); }}, "100%");
        menu.addShortCut("View/Magnification/100%", "ctrl+1");
        menu.addShortCut("View/Magnification/300%", "ctrl+3");
        menu.addShortCut("View/Magnification/500%", "ctrl+5");
        menu.addShortCut("View/Magnification/700%", "ctrl+7");
        menu.addShortCut("View/Magnification/1000%", "ctrl+0");
        menu.addItems("Modify/", new ArrayList<String>() {{ add("Flatten Layers"); add("Canvas"); add("Transform"); add("Arrange"); add("Align"); add("Selection"); }});
        menu.addShortCut("Modify/Flatten Layers", "ctrl+shift+z");
        menu.addItems("Modify/Canvas/", new ArrayList<String>(){{ add("Trim Canvas"); add("Fit Canvas"); add("Rotate 180°"); add("Rotate 90° CW"); add("Rotate 90° CCW"); }});
        menu.addItems("Modify/Transform/", new ArrayList<String>(){{ add("Rotate 180°"); add("Rotate 90° CW"); add("Rotate 90° CCW"); add("Flip Horizontal"); add("Flip Vertical"); }});
        menu.addItems("Modify/Arrange/", new ArrayList<String>(){{ add("Bring To Front"); add("Bring Forward"); add("Send Backward"); add("Send To Back"); }});
        menu.addShortCut("Modify/Arrange/Bring To Front", "ctrl+shift+up");
        menu.addShortCut("Modify/Arrange/Bring Forward", "ctrl+up");
        menu.addShortCut("Modify/Arrange/Send Backward", "ctrl+down");
        menu.addShortCut("Modify/Arrange/Send To Back", "ctrl+shift+down");
        menu.addItems("Modify/Align/", new ArrayList<String>(){{ add("Left"); add("Center Vertical"); add("Right"); add("Top"); add("Center Horizontal"); add("Bottom"); }});
        menu.addItems("Modify/Selection/", new ArrayList<String>(){{ add("New image via copy"); add("New image via cut"); }});
        menu.addItemsAsSet("Tools/", new ArrayList<String>(){{ add("Handle"); add("Draw"); add("Fill"); add("Wand"); add("Erase"); add("Eye Dropper"); }}, "Draw");
        menu.addShortCut("Tools/Handle", "ctrl+h");
        menu.addShortCut("Tools/Draw", "ctrl+d");
        menu.addShortCut("Tools/Fill", "ctrl+f");
        menu.addShortCut("Tools/Wand", "ctrl+g");
        menu.addShortCut("Tools/Erase", "ctrl+e");
        menu.addShortCut("Tools/Eye Dropper", "ctrl+i");
        addObject(menu, 0, 0);
        if (start)
            Greenfoot.start();
        start = false;
    }
    GreenfootImage pic;
    public void act()
    {
        pic = new GreenfootImage(size.getX() * zoom < 440 ? (int)size.getX() * zoom : 440, size.getY() * zoom < 367 ? (int)size.getY() * zoom : 367);
        
        count += 5;
        if (count == 510)
            count = 0;
        
        GreenfootImage trans = new GreenfootImage(zoom == 1 ? 12 : zoom * 4, zoom == 1 ? 12 : zoom * 4);
        trans.setColor(Color.WHITE);
        trans.fill();
        trans.setColor(new Color(222, 222, 222));
        trans.fillRect(0, 0, trans.getWidth() / 2, trans.getHeight() / 2);
        trans.fillRect(trans.getWidth() / 2, trans.getHeight() / 2, trans.getWidth() / 2, trans.getHeight() / 2);
        for (int i = -(int)scroll.getX() % trans.getWidth(); i < pic.getWidth(); i += trans.getWidth())
            for (int k = -(int)scroll.getY() % trans.getHeight(); k < pic.getHeight(); k += trans.getHeight())
                pic.drawImage(trans, i, k);
        
        if (++soCount == 49)
            soCount = 1;
        
        int nn = -2 + soCount / 3 % 4 + 2;
        int[] soIndex = { nn, nn, nn, nn };
        for (int lI = layers.size() - 1; lI > -1; lI--)
        {
            Layer l = layers.get(lI);
            if (zoom == 1)
            {
                Point p = new Point((int)(l.getPosition().getX() - scroll.getX()), (int)(l.getPosition().getY() - scroll.getY()));
                pic.drawImage(l.getImage(), (int)p.getX(), (int)p.getY());
                if (tool == Tool.WAND && l.getSelected() != null)
                {
                    for (int y = 0; y < l.getHeight(); y++)
                    {
                        for (int x = 0; x < l.getWidth(); x++)
                        {
                            if (l.getSelected()[y][x])
                            {
                                int x_spot = (int)p.getX() + x;
                                int y_spot = (int)p.getY() + y;
                                if (x == 0 || !l.getSelected()[y][x - 1])
                                {
                                    if (x_spot > -1 && x_spot < pic.getWidth() && y_spot + 1 > -1 && y_spot + 1 < pic.getHeight())
                                        pic.setColorAt(x_spot, y_spot + 1, soIndex[3] < 2 ? Color.BLACK : Color.WHITE);
                                }
                                if (x == l.getImage().getWidth() - 1 || !l.getSelected()[y][x + 1])
                                {
                                    if (x_spot + zoom - 1 > -1 && x_spot + zoom - 1 < pic.getWidth() && y_spot + 1 > -1 && y_spot + 1 < pic.getHeight())
                                        pic.setColorAt(x_spot + zoom - 1, y_spot + 1, soIndex[1] < 2 ? Color.BLACK : Color.WHITE);
                                }
                                if (y == 0 || !l.getSelected()[y - 1][x])
                                {
                                    if (x_spot + 1 > -1 && x_spot + 1 < pic.getWidth() && y_spot > -1 && y_spot < pic.getHeight())
                                        pic.setColorAt(x_spot + 1, y_spot, soIndex[0] < 2 ? Color.BLACK : Color.WHITE);
                                    if (--soIndex[0] == -1)
                                        soIndex[0] = 3;
                                }
                                if (y == l.getImage().getHeight() - 1 || !l.getSelected()[y + 1][x])
                                {
                                    if (x_spot + 1 > -1 && x_spot + 1 < pic.getWidth() && y_spot + zoom - 1 > -1 && y_spot + zoom - 1 < pic.getHeight())
                                        pic.setColorAt(x_spot + 1, y_spot + zoom - 1, soIndex[2] < 2 ? Color.BLACK : Color.WHITE);
                                    if (++soIndex[2] == 4)
                                        soIndex[2] = 0;
                                }
                            }
                        }
                        if (++soIndex[3] == 4)
                            soIndex[3] = 0;
                        if (--soIndex[1] == -1)
                            soIndex[1] = 3;
                    }
                }
            }
            else
            {
                Point p = new Point((int)l.getPosition().getX() * zoom, (int)l.getPosition().getY() * zoom);
                Point br = new Point((int)(p.getX() + l.getWidth() * zoom), (int)(p.getY() + l.getHeight() * zoom));
                
                if (br.getX() > scroll.getX() && p.getX() < scroll.getX() + pic.getWidth())
                    if (br.getY() > scroll.getY() && p.getY() < scroll.getY() + pic.getHeight())
                    {
                        int startX = (p.getX() > scroll.getX()) ? 0 : (int)(scroll.getX() - p.getX()) / zoom;
                        int startY = (p.getY() > scroll.getY()) ? 0 : (int)(scroll.getY() - p.getY()) / zoom;
                        for (int y = startY; y < l.getHeight() && p.getY() + y * zoom - (int)scroll.getY() < pic.getHeight(); y++)
                        {
                            for (int x = startX; x < l.getWidth() && p.getX() + x * zoom - (int)scroll.getX() < pic.getWidth(); x++)
                            {
                                pic.setColor(l.getImage().getColorAt(x, y));
                                int x_spot = (int)p.getX() + x * zoom - (int)scroll.getX();
                                int y_spot = (int)p.getY() + y * zoom - (int)scroll.getY();
                                if (pic.getColor().getAlpha() != 0)
                                    pic.fillRect(x_spot, y_spot, zoom, zoom);
                                if (tool == Tool.WAND && l.getSelected() != null)
                                {
                                    pic.setColor(Color.GRAY);
                                    if (l.getSelected()[y][x])
                                    {
                                        if (x == 0 || !l.getSelected()[y][x - 1])
                                        {
                                            int soIndex3 = soIndex[3];
                                            for (int ii = 0; ii < zoom; ii++)
                                            {
                                                if (x_spot > -1 && x_spot < pic.getWidth() && y_spot + ii > -1 && y_spot + ii < pic.getHeight())
                                                    pic.setColorAt(x_spot, y_spot + ii, soIndex3 < 2 ? Color.BLACK : Color.WHITE);
                                                if (++soIndex3 == 4)
                                                    soIndex3 = 0;
                                            }
                                        }
                                        if (x == l.getImage().getWidth() - 1 || !l.getSelected()[y][x + 1])
                                        {
                                            int soIndex1 = soIndex[1];
                                            for (int ii = 0; ii < zoom; ii++)
                                            {
                                                if (x_spot + zoom - 1 > -1 && x_spot + zoom - 1 < pic.getWidth() && y_spot + ii > -1 && y_spot + ii < pic.getHeight())
                                                    pic.setColorAt(x_spot + zoom - 1, y_spot + ii, soIndex1 < 2 ? Color.BLACK : Color.WHITE);
                                                if (--soIndex1 == -1)
                                                    soIndex1 = 3;
                                            }
                                        }
                                        if (y == 0 || !l.getSelected()[y - 1][x])
                                            for (int ii = 0; ii < zoom; ii++)
                                            {
                                                if (x_spot + ii > -1 && x_spot + ii < pic.getWidth() && y_spot > -1 && y_spot < pic.getHeight())
                                                    pic.setColorAt(x_spot + ii, y_spot, soIndex[0] < 2 ? Color.BLACK : Color.WHITE);
                                                if (--soIndex[0] == -1)
                                                    soIndex[0] = 3;
                                            }
                                        if (y == l.getImage().getHeight() - 1 || !l.getSelected()[y + 1][x])
                                            for (int ii = 0; ii < zoom; ii++)
                                            {
                                                if (x_spot + ii > -1 && x_spot + ii < pic.getWidth() && y_spot + zoom - 1 > -1 && y_spot + zoom - 1 < pic.getHeight())
                                                    pic.setColorAt(x_spot + ii, y_spot + zoom - 1, soIndex[2] < 2 ? Color.BLACK : Color.WHITE);
                                                if (++soIndex[2] == 4)
                                                    soIndex[2] = 0;
                                            }
                                    }
                                }
                            }
                            for (int ii = 0; ii < zoom; ii++)
                            {
                                if (++soIndex[3] == 4)
                                    soIndex[3] = 0;
                                if (--soIndex[1] == -1)
                                    soIndex[1] = 3;
                            }
                        }
                    }
            }
        }
        
        int fd = count < 255 ? count : 510 - count;
        pic.setColor(new Color(fd, fd, fd > 255 ? fd - 255 : 255 - fd));
        
        for (int i = layers.size() - 1; i > -1; i--)
        {
            Point p = new Point((int)(lastMouse.getX() + scroll.getX()), (int)(lastMouse.getY() - 12 + scroll.getY()));
            if (listBox.getIndexes().contains(i) || (tool == Tool.HANDLE && !listBox.hasSelection() && lastPressedAt != null && layers.get(i).getPosition().getX() * zoom < Math.max(p.getX(), lastPressedAt.getX() * zoom) && layers.get(i).getRightSide() * zoom > Math.min(lastPressedAt.getX() * zoom, p.getX()) && layers.get(i).getPosition().getY() * zoom < Math.max(p.getY(), lastPressedAt.getY() * zoom) && layers.get(i).getBottomSide() * zoom > Math.min(lastPressedAt.getY() * zoom, p.getY())))
                pic.drawRect((int)(layers.get(i).getPosition().getX() * zoom - scroll.getX()) - 1, (int)(layers.get(i).getPosition().getY() * zoom - scroll.getY()) - 1, (int)layers.get(i).getImage().getWidth() * zoom + 1, (int)layers.get(i).getImage().getHeight() * zoom + 1);
        }
        if (tool == Tool.HANDLE && !listBox.hasSelection() && lastPressedAt != null)
        {
            pic.setColor(new Color(fd > 255 ? fd - 255 : 255 - fd, fd, fd));
            Point p = new Point((int)(lastMouse.getX() + scroll.getX()), (int)(lastMouse.getY() - 12 + scroll.getY()));
            pic.drawRect((int)(Math.min(p.getX(), lastPressedAt.getX() * zoom) - scroll.getX()), (int)(Math.min(p.getY(), lastPressedAt.getY() * zoom) - scroll.getY()), (int)Math.abs(p.getX() - lastPressedAt.getX() * zoom), (int)Math.abs(p.getY() - lastPressedAt.getY() * zoom));
        }
        else if (tool == Tool.ERASE)
        {
            pic.setColor(new Color(fd > 255 ? fd - 255 : 255 - fd, fd, fd));
            Point p = new Point((int)(lastMouse.getX() + scroll.getX()), (int)(lastMouse.getY() - 12 + scroll.getY()));
            pic.drawRect((int)(p.getX() - ((int)p.getX() % zoom) - (int)brushSlider.getValue() * zoom / 2 - scroll.getX()) + ((int)brushSlider.getValue() % 2 == 0 ? 0 : zoom / 2), (int)(p.getY() - ((int)p.getY() % zoom) - (int)brushSlider.getValue() * zoom / 2 - scroll.getY()) + ((int)brushSlider.getValue() % 2 == 0 ? 0 : zoom / 2), (int)brushSlider.getValue() * zoom, (int)brushSlider.getValue() * zoom);
        }
        
        boolean aTextBoxHasFocus = false;
        for (Object o : getObjects(TextBox.class))
            if (((TextBox)o).focused())
                aTextBoxHasFocus = true;
        MouseInfo mouse = Greenfoot.getMouseInfo();
        if (Greenfoot.mousePressed(this) && !aTextBoxHasFocus && !menu.isActive() && mouse.getX() > 0 && mouse.getX() < pic.getWidth() && mouse.getY() > 12 && mouse.getY() < 12 + pic.getHeight())
        {
            Point p = new Point((mouse.getX() + (int)scroll.getX()) / zoom, (mouse.getY() - 12 + (int)scroll.getY()) / zoom);
            switch (tool)
            {
                case HANDLE:    boolean h = listBox.hasSelection();
                                if (h)
                                {
                                    h = false;
                                    for (Integer i : listBox.getIndexes())
                                        if (p.getX() > layers.get(i).getPosition().getX() && p.getX() < layers.get(i).getRightSide() && p.getY() > layers.get(i).getPosition().getY() && p.getY() < layers.get(i).getBottomSide())
                                        {
                                            h = true;
                                            break;
                                        }
                                }
                                if (!h)
                                {
                                    if (Greenfoot.isKeyDown("shift"))
                                    {
                                        int n = findTopLayerAt((int)p.getX(), (int)p.getY());
                                        if (n != -1)
                                            listBox.addIndex(n);
                                        else
                                            listBox.setIndex(findTopLayerAt((int)p.getX(), (int)p.getY()));
                                    }
                                    else
                                        listBox.setIndex(findTopLayerAt((int)p.getX(), (int)p.getY()));
                                }
                                else if (Greenfoot.isKeyDown("shift"))
                                    listBox.getIndexes().remove(new Integer(findTopLayerAt((int)p.getX(), (int)p.getY())));
                                if (listBox.hasSelection())
                                    lastPressedAt = p;
                                break;
                case DRAW:      lastPressedAt = p;
                                if (listBox.hasSelection())
                                {
                                    layers.get(listBox.getOneIndex()).drawAt((int)lastPressedAt.getX(), (int)lastPressedAt.getY(), (int)brushSlider.getValue(), drawingColor);
                                    propertyPanel.update((int)layers.get(listBox.getOneIndex()).getPosition().getX(), (int)layers.get(listBox.getOneIndex()).getPosition().getY(), layers.get(listBox.getOneIndex()).getWidth(), layers.get(listBox.getOneIndex()).getHeight());
                                }
                                else
                                {
                                    layers.add(0, new Layer(lastPressedAt, (int)brushSlider.getValue(), drawingColor));
                                    listBox.addStringAt(listBox.getSize() + "", 0);
                                }
                                break;
                case FILL:      if (listBox.hasSelection())
                                    layers.get(listBox.getOneIndex()).fillAt((int)p.getX(), (int)p.getY(), drawingColor);
                                break;
                case WAND:      if (listBox.hasSelection())
                                    layers.get(listBox.getOneIndex()).wandPressedAt((int)p.getX(), (int)p.getY(), (int)toleranceSlider.getValue());
                                break;
                case ERASE:     lastPressedAt = p;
                                if (listBox.hasSelection())
                                {
                                    if (layers.get(listBox.getOneIndex()).erase((int)p.getX(), (int)p.getY(), (int)brushSlider.getValue()))
                                    {
                                        layers.remove(listBox.getOneIndex());
                                        listBox.removeSelected();
                                    }
                                    else
                                        propertyPanel.update((int)layers.get(listBox.getOneIndex()).getPosition().getX(), (int)layers.get(listBox.getOneIndex()).getPosition().getY(), layers.get(listBox.getOneIndex()).getWidth(), layers.get(listBox.getOneIndex()).getHeight());
                                }
                                break;
                case DROPPER:   lastPressedAt = p;
                                if (listBox.hasSelection())
                                {
                                    Color c = layers.get(listBox.getOneIndex()).getColorAt((int)p.getX(), (int)p.getY());
                                    if (c != null)
                                    {
                                        drawingColor = c;
                                        myPalette.setColor(c);
                                    }
                                }
            }
        }
        if (Greenfoot.mouseDragged(null) || Greenfoot.mouseMoved(null))
            lastMouse = new Point(mouse.getX(), mouse.getY());
        if (Greenfoot.mouseDragged(null))
        {
            Point p = new Point((mouse.getX() + (int)scroll.getX()) / zoom, (mouse.getY() - 12 + (int)scroll.getY()) / zoom);
            if (listBox.hasSelection())
            {
                switch (tool)
                {
                    case HANDLE:    if (lastPressedAt != null)
                                    {
                                        for (Integer i : listBox.getIndexes())
                                            layers.get(i).getPosition().translate((int)(p.getX() - lastPressedAt.getX()), (int)(p.getY() - lastPressedAt.getY()));
                                        propertyPanel.translate((int)(p.getX() - lastPressedAt.getX()), (int)(p.getY() - lastPressedAt.getY()), 0, 0);
                                        lastPressedAt = p;
                                    }
                                    break;
                    case DRAW:      if (lastPressedAt != null)
                                    {
                                        layers.get(listBox.getOneIndex()).drawBetween(lastPressedAt, p, (int)brushSlider.getValue(), drawingColor);
                                        propertyPanel.update((int)layers.get(listBox.getOneIndex()).getPosition().getX(), (int)layers.get(listBox.getOneIndex()).getPosition().getY(), layers.get(listBox.getOneIndex()).getWidth(), layers.get(listBox.getOneIndex()).getHeight());
                                        lastPressedAt = p;
                                    }
                                    break;
                    case ERASE:     if (lastPressedAt != null)
                                    {
                                        if (layers.get(listBox.getOneIndex()).eraseBetween(lastPressedAt, p, (int)brushSlider.getValue()))
                                        {
                                            layers.remove(listBox.getOneIndex());
                                            listBox.removeSelected();
                                        }
                                        else
                                            propertyPanel.update((int)layers.get(listBox.getOneIndex()).getPosition().getX(), (int)layers.get(listBox.getOneIndex()).getPosition().getY(), layers.get(listBox.getOneIndex()).getWidth(), layers.get(listBox.getOneIndex()).getHeight());
                                        lastPressedAt = p;
                                    }
                                    break;
                }
            }
            else if (tool == Tool.HANDLE && lastPressedAt == null)
                lastPressedAt = p;
        }
        if (Greenfoot.mouseClicked(null) || Greenfoot.mouseDragEnded(null))
        {
            if (!listBox.hasSelection() && tool == Tool.HANDLE && lastPressedAt != null)
            {
                Point p = new Point((mouse.getX() + (int)scroll.getX()), (mouse.getY() - 12 + (int)scroll.getY()));
                for (int i = 0; i < layers.size(); i++)
                    if (layers.get(i).getPosition().getX() * zoom < Math.max(p.getX(), lastPressedAt.getX() * zoom) && layers.get(i).getRightSide() * zoom > Math.min(lastPressedAt.getX() * zoom, p.getX()) && layers.get(i).getPosition().getY() * zoom < Math.max(p.getY(), lastPressedAt.getY() * zoom) && layers.get(i).getBottomSide() * zoom > Math.min(lastPressedAt.getY() * zoom, p.getY()))
                        listBox.addIndex(i);
            }
            lastPressedAt = null;
            dragScrollBarX = -1;
            dragScrollBarY = -1;
        }
        
        GreenfootImage layout = new GreenfootImage(600, 400);
        layout.setColor(Color.WHITE);
        layout.fill();
        layout.setColor(Color.BLACK);
        layout.drawRect(-1, 11, pic.getWidth() + 1, pic.getHeight() + 1);
        layout.drawImage(pic, 0, 12);
        layout.drawRect(455, 31, 10, 10);
        
        layout.setColor(drawingColor);
        layout.fillRect(456, 32, 9, 9);
        if (Greenfoot.mousePressed(this) && myPalette.getWorld() == null && mouse.getX() > 455 && mouse.getX() < 465 + 2 && mouse.getY() > 31 && mouse.getY() < 41 + 2)
            addObject(myPalette, 300, 200);
        if (myPalette.hasColorReady())
            drawingColor = new Color(myPalette.getColor().getRed(), myPalette.getColor().getGreen(), myPalette.getColor().getBlue(), myPalette.getColor().getAlpha());
        
        if (Greenfoot.mousePressed(this) && mouse.getX() > 455 && mouse.getX() < 471 && mouse.getY() > 12 && mouse.getY() < 28)
            menu.provokeItem("Tools/Handle");
        if (Greenfoot.mousePressed(this) && mouse.getX() > 475 && mouse.getX() < 491 && mouse.getY() > 12 && mouse.getY() < 28)
            menu.provokeItem("Tools/Draw");
        if (Greenfoot.mousePressed(this) && mouse.getX() > 495 && mouse.getX() < 511 && mouse.getY() > 12 && mouse.getY() < 28)
            menu.provokeItem("Tools/Fill");
        if (Greenfoot.mousePressed(this) && mouse.getX() > 515 && mouse.getX() < 531 && mouse.getY() > 12 && mouse.getY() < 28)
            menu.provokeItem("Tools/Wand");
        if (Greenfoot.mousePressed(this) && mouse.getX() > 535 && mouse.getX() < 551 && mouse.getY() > 12 && mouse.getY() < 28)
            menu.provokeItem("Tools/Erase");
        if (Greenfoot.mousePressed(this) && mouse.getX() > 555 && mouse.getX() < 571 && mouse.getY() > 12 && mouse.getY() < 28)
            menu.provokeItem("Tools/Eye Dropper");
        
        layout.setColor(Color.RED);
        if (tool == Tool.HANDLE)
            layout.drawRect(455 - 1, 12 - 1, 16 + 2, 16 + 2);
        layout.drawImage(toolMovePic, 455, 12);
        if (tool == Tool.DRAW)
            layout.drawRect(475 - 1, 12 - 1, 16 + 2, 16 + 2);
        layout.drawImage(toolDrawPic, 475, 12);
        if (tool == Tool.FILL)
            layout.drawRect(495 - 1, 12 - 1, 16 + 2, 16 + 2);
        layout.drawImage(toolFillPic, 495, 12);
        if (tool == Tool.WAND)
            layout.drawRect(515 - 1, 12 - 1, 16 + 2, 16 + 2);
        layout.drawImage(toolWandPic, 515, 12);
        if (tool == Tool.ERASE)
            layout.drawRect(535 - 1, 12 - 1, 16 + 2, 16 + 2);
        layout.drawImage(toolErasePic, 535, 12);
        if (tool == Tool.DROPPER)
            layout.drawRect(555 - 1, 12 - 1, 16 + 2, 16 + 2);
        layout.drawImage(toolDropperPic, 555, 12);
        
        layout.drawString(tool == Tool.DRAW ? "Brush Size" : tool == Tool.ERASE ? "Eraser Size" : tool == Tool.WAND ? "Tolerance" : "", 473, 76);
        
        int xScrollBarWidth = 0;
        if (size.getX() * zoom > 440)
        {
            double ratio = ((440d / zoom) / size.getX());
            xScrollBarWidth = (int)(((440d / zoom) / size.getX()) * 440d);
            layout.setColor(Color.RED);
            layout.fillRect(scrollBarX, 380, xScrollBarWidth, 7);
            if (Greenfoot.mousePressed(this) && mouse.getX() > scrollBarX && mouse.getX() < scrollBarX + xScrollBarWidth && mouse.getY() > 380 - 2 && mouse.getY() < 387 + 2)
                dragScrollBarX = mouse.getX();
        }
        int yScrollBarLength = 0;
        if (size.getY() * zoom > 368)
        {
            yScrollBarLength = (int)(368 / zoom / size.getY() * 368);
            layout.setColor(Color.RED);
            layout.fillRect(441, 11 + scrollBarY, 7, yScrollBarLength);
            if (Greenfoot.mousePressed(this) && mouse.getX() > 441 - 2 && mouse.getX() < 448 + 2 && mouse.getY() > 11 + scrollBarY && mouse.getY() < 11 + scrollBarY + yScrollBarLength)
                dragScrollBarY = mouse.getY();
        }
        
        if (!listBox.hasSelection() && tool == Tool.HANDLE && lastPressedAt != null)
        {
            if (lastMouse.getX() < 0)
            {
                scrollBarX -= 2;
                if (scrollBarX < 0)
                    scrollBarX = 0;
                scroll = new Point((int)(scrollBarX / 440d * (size.getX() * zoom)), (int)scroll.getY());
            }
            else if (size.getX() * zoom > 440 && lastMouse.getX() > 440)
            {
                scrollBarX += 2;
                xScrollBarWidth = (int)(((440d / zoom) / size.getX()) * 440d);
                if (scrollBarX > 440 - xScrollBarWidth)
                    scrollBarX = 440 - xScrollBarWidth;
                scroll = new Point((int)(scrollBarX / 440d * (size.getX() * zoom)), (int)scroll.getY());
            }
            if (lastMouse.getY() < 12)
            {
                scrollBarY -= 2;
                if (scrollBarY < 0)
                    scrollBarY = 0;
                scroll = new Point((int)scroll.getX(), (int)(scrollBarY / 368d * (size.getY() * zoom)));
            }
            else if (size.getY() * zoom > 368 && lastMouse.getY() > 368 + 12)
            {
                scrollBarY += 2;
                yScrollBarLength = (int)(368 / zoom / size.getY() * 368);
                if (scrollBarY > 368 - yScrollBarLength)
                    scrollBarY = 368 - yScrollBarLength;
                scroll = new Point((int)scroll.getX(), (int)(scrollBarY / 368d * (size.getY() * zoom)));
            }
        }
        
        if (tool == Tool.DROPPER && listBox.hasSelection() && lastMouse.getX() < pic.getWidth() && lastMouse.getY() < 12 + pic.getHeight())
        {
            Color c = layers.get(listBox.getOneIndex()).getColorAt(((int)lastMouse.getX() + (int)scroll.getX()) / zoom, ((int)lastMouse.getY() - 12 + (int)scroll.getY()) / zoom);
            if (c != null)
            {
                layout.setColor(c);
                layout.fillRect((int)lastMouse.getX(), (int)lastMouse.getY(), 25, 25);
                layout.setColor(Color.BLACK);
                layout.drawRect((int)lastMouse.getX(), (int)lastMouse.getY(), 25, 25);
            }
        }
        
        setBackground(layout);
        
        if (Greenfoot.mouseDragged(null))
        {
            if (dragScrollBarX != -1)
            {
                scrollBarX += (mouse.getX() - dragScrollBarX);
                dragScrollBarX = mouse.getX();
                if (scrollBarX < 0)
                    scrollBarX = 0;
                else if (scrollBarX > 440 - xScrollBarWidth)
                    scrollBarX = 440 - xScrollBarWidth;
                scroll = new Point((int)(scrollBarX / 440d * (size.getX() * zoom)), (int)scroll.getY());
            }
            else if (dragScrollBarY != -1)
            {
                scrollBarY += (mouse.getY() - dragScrollBarY);
                dragScrollBarY = mouse.getY();
                if (scrollBarY < 0)
                    scrollBarY = 0;
                else if (scrollBarY > 368 - yScrollBarLength)
                    scrollBarY = 368 - yScrollBarLength;
                scroll = new Point((int)scroll.getX(), (int)(scrollBarY / 368d * (size.getY() * zoom)));
            }
        }
        
        boolean overListBox = false;
        for (Object o : getObjects(ListBox.class))
        {
            ListBox lb = (ListBox)o;
            if (lastMouse.getX() > lb.getX() - lb.getImage().getWidth() / 2 && lastMouse.getX() < lb.getX() + lb.getImage().getWidth() / 2 && lastMouse.getY() > lb.getY() - lb.getImage().getHeight() / 2 && lastMouse.getY() < lb.getY() + lb.getImage().getHeight() / 2)
                overListBox = true;
        }
        int n = scroller.getScroll();
        if (!overListBox && (fileHandler.getWorld() == null || lastMouse.getX() < fileHandler.getX() - fileHandler.getImage().getWidth() / 2 || lastMouse.getX() > fileHandler.getX() + fileHandler.getImage().getWidth() / 2 || lastMouse.getY() < fileHandler.getY() - fileHandler.getImage().getHeight() / 2 || lastMouse.getY() > fileHandler.getY() + fileHandler.getImage().getHeight() / 2))
            if (size.getY() * zoom > 368)
            {
                scrollBarY += n;
                if (scrollBarY < 0)
                    scrollBarY = 0;
                else if (scrollBarY > 368 - yScrollBarLength)
                    scrollBarY = 368 - yScrollBarLength;
                scroll = new Point((int)scroll.getX(), (int)(scrollBarY / 368d * (size.getY() * zoom)));
            }
        
        if (listBox.hasChanged())
            checkProperties();
        
        if (listBox.getIndexes().size() > 1 && (tool == Tool.DRAW || tool == Tool.FILL || tool == Tool.WAND || tool == Tool.ERASE || tool == Tool.DROPPER))
            menu.provokeItem("Tools/Handle");
        
        menu.enableItem("Edit/Cut", listBox.hasSelection());
        menu.enableItem("Edit/Copy", listBox.hasSelection());
        menu.enableItem("Edit/Paste", clipBoard.size() > 0);
        menu.enableItem("Edit/Clear", !aTextBoxHasFocus);
        menu.enableItem("View/Zoom In", zoom < 10);
        menu.enableItem("View/Zoom Out", zoom > 1);
        
        menu.enableItem("Modify/Canvas/Trim Canvas", layers.size() > 0);
        menu.enableItem("Modify/Canvas/Fit Canvas", layers.size() > 0);
        menu.enableItem("Modify/Flatten Layers", listBox.getIndexes().size() > 1);
        menu.enableItem("Modify/Transform", listBox.hasSelection());
        menu.enableItem("Modify/Arrange", listBox.hasSelection());
        menu.enableItem("Modify/Align", listBox.getIndexes().size() > 1);
        menu.enableItem("Modify/Selection", tool == Tool.WAND && listBox.hasSelection());
        
        listenToMenu();
        
        if (sizeInputPanel.hasSizeReady())
        {
            size = sizeInputPanel.getSize();
            listBox.clear();
            layers.clear();
            scroll = new Point(0, 0);
            scrollBarX = 0;
            scrollBarY = 0;
            menu.provokeItem("View/Magnification/100%");
        }
        if (fileHandler.isReady())
        {
            if (fileHandler.isLoading())
            {
                GreenfootImage g = fileHandler.getImageLoaded();
                listBox.clear();
                layers.clear();
                size = new Point(g.getWidth(), g.getHeight());
                layers.add(new Layer(g, new Point(0, 0)));
                if (layers.get(0).fitBounds())
                    layers.clear();
                else
                    listBox.addString("0");
                scroll = new Point(0, 0);scrollBarX = 0;
                scrollBarY = 0;
                menu.provokeItem("View/Magnification/100%");
            }
            else
            {
                GreenfootImage image = new GreenfootImage((int)size.getX(), (int)size.getY());
                for (int i = layers.size() - 1; i > -1; i--)
                    image.drawImage(layers.get(i).getImage(), (int)layers.get(i).getPosition().getX(), (int)layers.get(i).getPosition().getY());
                fileHandler.save(image);
            }
        }
        for (int i = 0; i < 10; i++)
            if (Greenfoot.isKeyDown(i + ""))
            {}
        for (int i = 'a'; i <= 'z'; i++)
            if (Greenfoot.isKeyDown((char)i + ""))
            {}
        for (int i = 1; i < 13; i++)
            if (Greenfoot.isKeyDown("F" + i))
            {}
        for (String s = "; ' [ ] ` / \\ , . - = up down right left control shift enter space tab escape backspace "; s.length() > 0; s = s.substring(s.indexOf(" ") + 1))
            if (Greenfoot.isKeyDown(s.substring(0, s.indexOf(" "))))
            {}
        
        if (!shownCode && layers.size() > 4)
        {
            shownCode = true;
            if (CodeGenerator.haveUser())
                addObject(new CodeDisplay(CODE_ID), getWidth() / 2, getHeight() - 6);
        }
    }
    private int dragScrollBarX = -1;
    private int scrollBarX = 0;
    private int dragScrollBarY = -1;
    private int scrollBarY = 0;
    
    private int findTopLayerAt(int x, int y)
    {
        for (int i = 0; i < layers.size(); i++)
        {
            int xx = x - (int)(layers.get(i).getPosition().getX());
            int yy = y - (int)(layers.get(i).getPosition().getY());
            if (xx > -1 && xx < layers.get(i).getImage().getWidth() && yy > -1 && yy < layers.get(i).getImage().getHeight())
                if (layers.get(i).getImage().getColorAt(xx, yy).getAlpha() != 0)
                    return i;
        }
        return -1;
    }
    
    private void listenToMenu()
    {
        String s = menu.getItemPressed();
        if (s != null)
        {
            if (s.startsWith("File/"))
            {
                s = s.substring(s.indexOf("/") + 1);
                boolean panelsClosed = sizeInputPanel.getWorld() == null && fileHandler.getWorld() == null;
                if (panelsClosed)
                    if (s.equals("New/"))
                        addObject(sizeInputPanel, 300, 200);
                    else if (s.equals("Open/"))
                    {
                        fileHandler.load(true);
                        addObject(fileHandler, 300, 200);
                    }
                    else if (s.equals("Save/"))
                    {
                        fileHandler.load(false);
                        addObject(fileHandler, 300, 200);
                    }
            }
            else if (s.startsWith("Edit/"))
            {
                s = s.substring(s.indexOf("/") + 1);
                if (s.equals("Cut/"))
                {
                    clipBoard = new ArrayList<Layer>();
                    for (int i = 0; i < layers.size(); i++)
                        if (listBox.getIndexes().contains(i))
                            clipBoard.add(new Layer(layers.get(i)));
                    ArrayList<Layer> layerD = new ArrayList<Layer>();
                    for (Integer i : listBox.getIndexes())
                        layerD.add(layers.get(i));
                    for (Layer l : layerD)
                        layers.remove(l);
                    listBox.removeSelected();
                }
                else if (s.equals("Copy/"))
                {
                    clipBoard = new ArrayList<Layer>();
                    for (int i = 0; i < layers.size(); i++)
                        if (listBox.getIndexes().contains(i))
                            clipBoard.add(new Layer(layers.get(i)));
                }
                else if (s.equals("Paste/"))
                {
                    int n;
                    if (!listBox.hasSelection())
                        n = 0;
                    else
                        for (n = 0; !listBox.getIndexes().contains(n); n++)
                        {}
                    for (int i = clipBoard.size() - 1; i > -1; i--)
                    {
                        layers.add(n, new Layer(clipBoard.get(i)));
                        listBox.addStringAt(layers.size() + "", n);
                    }
                    for (int i = n; i < n + clipBoard.size(); i++)
                        listBox.addIndex(i);
                }
                else if (s.equals("Clear/"))
                {
                    if (tool == Tool.WAND)
                    {
                        if (listBox.hasSelection())
                            layers.get(listBox.getOneIndex()).deleteSelected();
                    }
                    else
                    {
                        ArrayList<Layer> layerD = new ArrayList<Layer>();
                        for (Integer i : listBox.getIndexes())
                            layerD.add(layers.get(i));
                        for (Layer l : layerD)
                            layers.remove(l);
                        listBox.removeSelected();
                    }
                }
                else if (s.equals("Select All/"))
                {
                    if (tool == Tool.WAND)
                        layers.get(listBox.getOneIndex()).selectAll();
                    else
                        listBox.selectAll();
                }
            }
            else if (s.startsWith("View/"))
            {
                s = s.substring(s.indexOf("/") + 1);
                if (s.equals("Zoom In/"))
                {
//                     if (zoom < 10)
//                     {
                        int n = 0;
                        switch (zoom)
                        {
                            case 1: n = 3; break;
                            case 3: n = 5; break;
                            case 5: n = 7; break;
                            case 7: n = 10;
                        }
                        menu.provokeItem("View/Magnification/" + n * 100 + "%");
//                     }
                }
                else if (s.equals("Zoom Out/"))
                {
//                     if (zoom > 1)
//                     {
                        int n = 0;
                        switch (zoom)
                        {
                            case 10: n = 7; break;
                            case 7: n = 5; break;
                            case 5: n = 3; break;
                            case 3: n = 1;
                        }
                        menu.provokeItem("View/Magnification/" + n * 100 + "%");
//                     }
                }
                else if (s.startsWith("Magnification/"))
                {
                    s = s.substring(s.indexOf("/") + 1);
                    if (s.length() > 0)
                    {
                        int oldZ = zoom;
                        zoom = Integer.parseInt(s.substring(0, s.indexOf("%"))) / 100;
                        if (listBox.hasSelection())
                        {
                            Point p1 = null;
        //                     Point p2 = null;
                            for (int i : listBox.getIndexes())
                            {
                                if (p1 == null)
                                    p1 = new Point(layers.get(i).getPosition());
                                else
                                {
                                    if (layers.get(i).getPosition().getX() < p1.getX())
                                        p1 = new Point((int)layers.get(i).getPosition().getX(), (int)p1.getY());
                                    if (layers.get(i).getPosition().getY() < p1.getY())
                                        p1 = new Point((int)p1.getX(), (int)layers.get(i).getPosition().getY());
                                }
        //                         if (p2 == null)
        //                             p2 = new Point(layers.get(i).getRightSide(), layers.get(i).getBottomSide());
        //                         else
        //                         {
        //                             if (layers.get(i).getRightSide() < p2.getX())
        //                                 p2 = new Point((int)layers.get(i).getRightSide(), (int)p2.getY());
        //                             if (layers.get(i).getBottomSide() < p2.getY())
        //                                 p2 = new Point((int)p2.getX(), (int)layers.get(i).getBottom());
        //                         }
                            }
                            if (p1.getX() < 0)
                                p1 = new Point(0, (int)p1.getY());
                            if (p1.getY() < 0)
                                p1 = new Point((int)p1.getX(), 0);
                            scroll = new Point((int)p1.getX() * zoom, (int)p1.getY() * zoom);
                            if (size.getX() * zoom < 440)
                            {
                                scroll = new Point(0, (int)scroll.getY());
                                scrollBarX = 0;
                            }
                            else
                            {
                                if (scroll.getX() > (440 - (int)(((440d / zoom) / size.getX()) * 440d)) / 440d * (size.getX() * zoom))
                                    scroll = new Point((int)((440 - (int)(((440d / zoom) / size.getX()) * 440d)) / 440d * (size.getX() * zoom)), (int)scroll.getY());
                                scrollBarX = (int)(scroll.getX() * 440d / (size.getX() * zoom));
                            }
                            if (size.getY() * zoom < 368)
                            {
                                scroll = new Point((int)scroll.getX(), 0);
                                scrollBarY = 0;
                            }
                            else
                            {
                                if (scroll.getY() > (368 - (int)(368 / zoom / size.getY() * 368)) / 368d * (size.getY() * zoom))
                                    scroll = new Point((int)scroll.getX(), (int)((368 - (int)(368 / zoom / size.getY() * 368)) / 368d * (size.getY() * zoom)));
                                scrollBarY = (int)(scroll.getY() * 368d / (size.getY() * zoom));
                            }
                        }
                        else
                        {
                            scroll = new Point(0, 0);
                            scrollBarX = 0;
                            scrollBarY = 0;
                        }
                    }
                }
            }
            else if (s.startsWith("Modify/"))
            {
                s = s.substring(s.indexOf("/") + 1);
                if (s.equals("Flatten Layers/"))
                    flattenSelections();
                else if (s.startsWith("Canvas/"))
                {
                    s = s.substring(s.indexOf("/") + 1);
                    if (s.equals("Trim Canvas/") || s.equals("Fit Canvas/"))
                    {
                        if (s.equals("Trim Canvas/"))
                            for (int i = 0; i < layers.size(); i++)
                                if (layers.get(i).trimmingCanvas(size))
                                {
                                    listBox.remove(i);
                                    layers.remove(i--);
                                }
                        if (layers.size() > 0)
                        {
                            int xLow = layers.get(0).getX();
                            int yLow = layers.get(0).getY();
                            for (Layer l : layers)
                            {
                                if (l.getX() < xLow)
                                    xLow = l.getX();
                                if (l.getY() < yLow)
                                    yLow = l.getY();
                            }
                            int xMax = -1;
                            int yMax = -1;
                            for (Layer l : layers)
                            {
                                l.getPosition().translate(-xLow, -yLow);
                                if (l.getRightSide() > xMax)
                                    xMax = l.getRightSide();
                                if (l.getBottomSide() > yMax)
                                    yMax = l.getBottomSide();
                            }
                            size = new Point(xMax, yMax);
                            scroll = new Point(0, 0);
                            scrollBarX = 0;
                            scrollBarY = 0;
                            checkProperties();
                        }
                    }
                    else if (s.startsWith("Rotate "))
                    {
                        s = s.substring(s.indexOf(" ") + 1);
                        for (int i = s.equals("90° CW/") ? 1 : s.equals("180°/") ? 2 : 3; i > 0; i--)
                            rotateCW(true);
                        checkProperties();
                        scroll = new Point(0, 0);
                        scrollBarX = 0;
                        scrollBarY = 0;
                    }
                }
                else if (s.startsWith("Transform/"))
                {
                    s = s.substring(s.indexOf("/") + 1);
                    if (s.startsWith("Rotate "))
                    {
                        s = s.substring(s.indexOf(" ") + 1);
                        for (int i = s.equals("90° CW/") ? 1 : s.equals("180°/") ? 2 : 3; i > 0; i--)
                            rotateCW(false);
                        checkProperties();
                    }
                    else if (s.equals("Flip Horizontal/"))
                    {
                        int low = layers.get(listBox.getOneIndex()).getX();
                        int high = layers.get(listBox.getOneIndex()).getRightSide();
                        for (int i : listBox.getIndexes())
                        {
                            if (layers.get(i).getX() < low)
                                low = layers.get(i).getX();
                            if (layers.get(i).getRightSide() > high)
                                high = layers.get(i).getRightSide();
                        }
                        for (int i : listBox.getIndexes())
                        {
                            if (layers.get(i).getX() - low < high - layers.get(i).getRightSide())
                                layers.get(i).setPosition(new Point(high - layers.get(i).getWidth() - (layers.get(i).getX() - low), layers.get(i).getY()));
                            else
                                layers.get(i).setPosition(new Point(low + high - layers.get(i).getRightSide(), layers.get(i).getY()));
                            layers.get(i).getImage().mirrorHorizontally();
                        }
                    }
                    else if (s.equals("Flip Vertical/"))
                    {
                        int low = layers.get(listBox.getOneIndex()).getY();
                        int high = layers.get(listBox.getOneIndex()).getBottomSide();
                        for (int i : listBox.getIndexes())
                        {
                            if (layers.get(i).getY() < low)
                                low = layers.get(i).getY();
                            if (layers.get(i).getBottomSide() > high)
                                high = layers.get(i).getBottomSide();
                        }
                        for (int i : listBox.getIndexes())
                        {
                            if (layers.get(i).getY() - low < high - layers.get(i).getBottomSide())
                                layers.get(i).setPosition(new Point(layers.get(i).getX(), high - layers.get(i).getHeight() - (layers.get(i).getY() - low)));
                            else
                                layers.get(i).setPosition(new Point(layers.get(i).getX(), low + high - layers.get(i).getBottomSide()));
                            layers.get(i).getImage().mirrorVertically();
                        }
                    }
                }
                else if (s.startsWith("Arrange/"))
                {
                    s = s.substring(s.indexOf("/") + 1);
                    if (s.equals("Bring To Front/") || s.equals("Bring Forward/"))
                        changeLayerOrder(true, s.equals("Bring To Front/"));
                    else if (s.equals("Send Backward/") || s.equals("Send To Back/"))
                        changeLayerOrder(false, s.equals("Send To Back/"));
                }
                else if (s.startsWith("Align/"))
                {
                    s = s.substring(s.indexOf("/") + 1);
                    if (s.equals("Left/"))
                    {
                        int low = layers.get(listBox.getOneIndex()).getX();
                        for (int i : listBox.getIndexes())
                            if (layers.get(i).getX() < low)
                                low = layers.get(i).getX();
                        for (int i : listBox.getIndexes())
                            layers.get(i).setPosition(new Point(low, layers.get(i).getY()));
                    }
                    else if (s.equals("Center Vertical/"))
                    {
                        int low = layers.get(listBox.getOneIndex()).getX();
                        int high = layers.get(listBox.getOneIndex()).getRightSide();
                        for (int i : listBox.getIndexes())
                        {
                            if (layers.get(i).getX() < low)
                                low = layers.get(i).getX();
                            if (layers.get(i).getRightSide() > high)
                                high = layers.get(i).getRightSide();
                        }
                        int mid = (high + low) / 2;
                        for (int i : listBox.getIndexes())
                            layers.get(i).setPosition(new Point(mid - layers.get(i).getWidth() / 2, layers.get(i).getY()));
                    }
                    else if (s.equals("Right/"))
                    {
                        int high = layers.get(listBox.getOneIndex()).getRightSide();
                        for (int i : listBox.getIndexes())
                            if (layers.get(i).getRightSide() > high)
                                high = layers.get(i).getRightSide();
                        for (int i : listBox.getIndexes())
                            layers.get(i).setPosition(new Point(high - layers.get(i).getWidth(), layers.get(i).getY()));
                    }
                    else if (s.equals("Top/"))
                    {
                        int low = layers.get(listBox.getOneIndex()).getY();
                        for (int i : listBox.getIndexes())
                            if (layers.get(i).getY() < low)
                                low = layers.get(i).getY();
                        for (int i : listBox.getIndexes())
                            layers.get(i).setPosition(new Point(layers.get(i).getX(), low));
                    }
                    else if (s.equals("Center Horizontal/"))
                    {
                        int low = layers.get(listBox.getOneIndex()).getY();
                        int high = layers.get(listBox.getOneIndex()).getBottomSide();
                        for (int i : listBox.getIndexes())
                        {
                            if (layers.get(i).getY() < low)
                                low = layers.get(i).getY();
                            if (layers.get(i).getBottomSide() > high)
                                high = layers.get(i).getBottomSide();
                        }
                        int mid = (high + low) / 2;
                        for (int i : listBox.getIndexes())
                            layers.get(i).setPosition(new Point(layers.get(i).getX(), mid - layers.get(i).getHeight() / 2));
                    }
                    else if (s.equals("Bottom/"))
                    {
                        int high = layers.get(listBox.getOneIndex()).getBottomSide();
                        for (int i : listBox.getIndexes())
                            if (layers.get(i).getBottomSide() > high)
                                high = layers.get(i).getBottomSide();
                        for (int i : listBox.getIndexes())
                            layers.get(i).setPosition(new Point(layers.get(i).getX(), high - layers.get(i).getHeight()));
                    }
                    if (s.length() != 0)
                    {
                        int xP = (int)layers.get(listBox.getOneIndex()).getPosition().getX();
                        int rP = (int)layers.get(listBox.getOneIndex()).getRightSide();
                        int yP = (int)layers.get(listBox.getOneIndex()).getPosition().getY();
                        int bP = (int)layers.get(listBox.getOneIndex()).getBottomSide();
                        for (int i = 1; i < listBox.getIndexes().size(); i++)
                        {
                            int tempP = (int)layers.get(listBox.getIndexes().get(i)).getPosition().getX();
                            if (tempP < xP)
                                xP = tempP;
                            tempP = (int)layers.get(listBox.getIndexes().get(i)).getPosition().getY();
                            if (tempP < yP)
                                yP = tempP;
                            tempP = layers.get(listBox.getIndexes().get(i)).getRightSide();
                            if (tempP > rP)
                                rP = tempP;
                            tempP = layers.get(listBox.getIndexes().get(i)).getBottomSide();
                            if (tempP > bP)
                                bP = tempP;
                        }
                        propertyPanel.update(xP, yP, rP - xP, bP - yP);
                    }
                }
                else if (s.startsWith("Selection/"))
                {
                    s = s.substring(s.indexOf("/") + 1);
                    if (s.equals("New image via copy/") || s.equals("New image via cut/"))
                    {
                        if (tool == Tool.WAND && listBox.hasSelection())
                        {
                            boolean cutting = s.equals("New image via cut/");
                            Layer l = layers.get(listBox.getOneIndex()).newLayerFromSelected(cutting);
                            if (l != null)
                            {
                                layers.add(listBox.getOneIndex(), l);
                                listBox.addStringAt(listBox.getSize() + "", listBox.getOneIndex());
                                menu.provokeItem("Tools/Handle");
                            }
                        }
                    }
                }
            }
            else if (s.startsWith("Tools/"))
            {
                s = s.substring(s.indexOf("/") + 1);
                if (s.equals("Handle/"))
                {
                    beforeChangeTool(Tool.HANDLE);
                    tool = Tool.HANDLE;
                }
                else if (s.equals("Draw/"))
                {
                    beforeChangeTool(Tool.DRAW);
                    tool = Tool.DRAW;
                    if (listBox.getIndexes().size() > 1)
                        listBox.setIndex(-1);
                }
                else if (s.equals("Fill/"))
                {
                    beforeChangeTool(Tool.FILL);
                    tool = Tool.FILL;
                    if (listBox.getIndexes().size() > 1)
                        listBox.setIndex(-1);
                }
                else if (s.equals("Wand/"))
                {
                    beforeChangeTool(Tool.WAND);
                    tool = Tool.WAND;
                    if (listBox.getIndexes().size() > 1)
                        listBox.setIndex(-1);
                }
                else if (s.equals("Erase/"))
                {
                    beforeChangeTool(Tool.ERASE);
                    tool = Tool.ERASE;
                    if (listBox.getIndexes().size() > 1)
                        listBox.setIndex(-1);
                }
                else if (s.equals("Eye Dropper/"))
                {
                    beforeChangeTool(Tool.DROPPER);
                    tool = Tool.DROPPER;
                    if (listBox.getIndexes().size() > 1)
                        listBox.setIndex(-1);
                }
            }
        }
    }
    private void beforeChangeTool(Tool newT)
    {
        if (tool == newT)
            return;
        if (tool == Tool.WAND)
            if (listBox.hasSelection())
            {
                layers.get(listBox.getOneIndex()).deselect();
                if (layers.get(listBox.getOneIndex()).fitBounds())
                {
                    layers.remove(listBox.getOneIndex());
                    listBox.removeSelected();
                    listBox.setIndex(-1);
                }
            }
        if (newT == Tool.WAND)
            toleranceSlider.show();
        else if (tool == Tool.WAND)
            toleranceSlider.hide();
        if (newT == Tool.DRAW || newT == Tool.ERASE)
            brushSlider.show();
        else if (tool == Tool.DRAW || tool == Tool.ERASE)
            brushSlider.hide();
    }
    
    private void flattenSelections()
    {
        if (listBox.getIndexes().size() > 1)
        {
            int topL = -1;
            ArrayList<Layer> otherL = new ArrayList<Layer>();
            for (int i = 0; i < layers.size(); i++)
            {
                if (listBox.getIndexes().contains(i))
                {
                    if (topL == -1)
                        topL = i;
                    else
                        otherL.add(layers.get(i));
                }
            }
            layers.get(topL).flattenWith(otherL);
            for (Layer l : otherL)
                layers.remove(l);
            listBox.toggleSelect(topL);
            listBox.removeSelected();
            listBox.setIndex(topL);
            menu.provokeItem("Tools/Handle");
        }
    }
    
    private void rotateCW(boolean canvas)
    {
        int xLow = canvas ? 0 : layers.get(listBox.getOneIndex()).getX();
        int xHigh = canvas ? (int)size.getX() : layers.get(listBox.getOneIndex()).getRightSide();
        int yLow = canvas ? 0 : layers.get(listBox.getOneIndex()).getY();
        int yHigh = canvas ? (int)size.getY() : layers.get(listBox.getOneIndex()).getBottomSide();
        if (canvas)
            listBox.selectAll();
        for (int i : listBox.getIndexes())
        {
            if (layers.get(i).getX() < xLow)
                xLow = layers.get(i).getX();
            if (layers.get(i).getRightSide() > xHigh)
                xHigh = layers.get(i).getRightSide();
            if (layers.get(i).getY() < yLow)
                yLow = layers.get(i).getY();
            if (layers.get(i).getBottomSide() > yHigh)
                yHigh = layers.get(i).getBottomSide();
        }
        int newXLow = (xLow + xHigh) / 2 - Math.abs(yLow - yHigh) / 2;
        int newYLow = (yLow + yHigh) / 2 - Math.abs(xLow - xHigh) / 2;
        for (int i : listBox.getIndexes())
        {
            layers.get(i).setPosition(new Point(newXLow + yHigh - layers.get(i).getBottomSide() - (int)(canvas ? (size.getX() - size.getY()) / 2 : 0), newYLow + layers.get(i).getX() - xLow + (int)(canvas ? (size.getX() - size.getY()) / 2 : 0)));
            layers.get(i).rotateImageOneTurnClockWise();
        }
        if (canvas)
            size = new Point((int)size.getY(), (int)size.getX());
    }
    
    private void changeLayerOrder(boolean up, boolean shift)
    {
        if (!shift)
        {
            int num = 3;
        }
        else
        {
            int num = 2;
        }
        if (listBox.getIndexes().size() > 0)
        {
            int lastBumped = -1;
            if (up)
            {
                while (true)
                {
                    boolean swapped = false;
                    for (int i = 1; i < layers.size(); i++)
                    {
                        if (listBox.getIndexes().contains(new Integer(i)) && (!listBox.getIndexes().contains(new Integer(i - 1)) || lastBumped == i - 1))
                        {
                            listBox.swap(i, i - 1);
                            Layer l = layers.get(i - 1);
                            layers.set(i - 1, layers.get(i));
                            layers.set(i, l);
                            lastBumped = i;
                            swapped = true;
                        }
                    }
                    if (!shift || !swapped)
                        break;
                }
            }
            else
            {
                while (true)
                {
                    boolean swapped = false;
                    for (int i = layers.size() - 2; i > -1; i--)
                    {
                        if (listBox.getIndexes().contains(new Integer(i)) && (!listBox.getIndexes().contains(new Integer(i + 1)) || lastBumped == i + 1))
                        {
                            listBox.swap(i, i + 1);
                            Layer l = layers.get(i + 1);
                            layers.set(i + 1, layers.get(i));
                            layers.set(i, l);
                            lastBumped = i;
                            swapped = true;
                        }
                    }
                    if (!shift || !swapped)
                        break;
                }
            }
        }
    }
    public void updateByProperty(int p, int v)
    {
        int xP, rP, yP, bP;
        switch (p)
        {
            case 0:     xP = (int)layers.get(listBox.getOneIndex()).getPosition().getX();
                        for (int i = 1; i < listBox.getIndexes().size(); i++)
                        {
                            int tempP = (int)layers.get(listBox.getIndexes().get(i)).getPosition().getX();
                            if (tempP < xP)
                                xP = tempP;
                        }
                        for (int i : listBox.getIndexes())
                            layers.get(i).getPosition().translate(v - xP, 0);
                        break;
            case 1:     yP = (int)layers.get(listBox.getOneIndex()).getPosition().getY();
                        for (int i = 1; i < listBox.getIndexes().size(); i++)
                        {
                            int tempP = (int)layers.get(listBox.getIndexes().get(i)).getPosition().getY();
                            if (tempP < yP)
                                yP = tempP;
                        }
                        for (int i : listBox.getIndexes())
                            layers.get(i).getPosition().translate(0, v - yP);
                        break;
            case 2:     xP = (int)layers.get(listBox.getOneIndex()).getPosition().getX();
                        rP = (int)layers.get(listBox.getOneIndex()).getRightSide();
                        for (int i = 1; i < listBox.getIndexes().size(); i++)
                        {
                            int tempP = (int)layers.get(listBox.getIndexes().get(i)).getPosition().getX();
                            if (tempP < xP)
                                xP = tempP;
                            tempP = layers.get(listBox.getIndexes().get(i)).getRightSide();
                            if (tempP > rP)
                                rP = tempP;
                        }
                        int newLeft = xP + ((rP - xP) - v) / 2;
                        for (int i : listBox.getIndexes())
                        {
                            double r = (double)layers.get(i).getWidth() / (rP - xP);
                            int w = (int)(r * v);
                            layers.get(i).scale(w, layers.get(i).getHeight());
                            layers.get(i).setPosition(new Point((int)(newLeft + ((layers.get(i).getPosition().getX() - xP) / (rP - xP)) * v), (int)layers.get(i).getPosition().getY()));
                        }
                        propertyPanel.translate(newLeft - xP, 0, 0, 0);
                        break;
            case 3:     yP = (int)layers.get(listBox.getOneIndex()).getPosition().getY();
                        bP = (int)layers.get(listBox.getOneIndex()).getBottomSide();
                        for (int i = 1; i < listBox.getIndexes().size(); i++)
                        {
                            int tempP = (int)layers.get(listBox.getIndexes().get(i)).getPosition().getY();
                            if (tempP < yP)
                                yP = tempP;
                            tempP = layers.get(listBox.getIndexes().get(i)).getBottomSide();
                            if (tempP > bP)
                                bP = tempP;
                        }
                        int newRight = yP + ((bP - yP) - v) / 2;
                        for (int i : listBox.getIndexes())
                        {
                            double r = (double)layers.get(i).getHeight() / (bP - yP);
                            int h = (int)(r * v);
                            layers.get(i).scale(layers.get(i).getWidth(), h);
                            layers.get(i).setPosition(new Point((int)layers.get(i).getPosition().getX(), (int)(newRight + ((layers.get(i).getPosition().getY() - yP) / (bP - yP)) * v)));
                        }
                        propertyPanel.translate(0, newRight - yP, 0, 0);
                        break;
        }
        if (tool == Tool.WAND)
            menu.provokeItem("Tools/Handle");
    }
    private void checkProperties()
    {
        if (listBox.hasSelection())
        {
            if (propertyPanel.getWorld() == null)
                addObject(propertyPanel, 523, 230);
            int xP = (int)layers.get(listBox.getOneIndex()).getPosition().getX();
            int rP = (int)layers.get(listBox.getOneIndex()).getRightSide();
            int yP = (int)layers.get(listBox.getOneIndex()).getPosition().getY();
            int bP = (int)layers.get(listBox.getOneIndex()).getBottomSide();
            for (int i = 1; i < listBox.getIndexes().size(); i++)
            {
                int tempP = (int)layers.get(listBox.getIndexes().get(i)).getPosition().getX();
                if (tempP < xP)
                    xP = tempP;
                tempP = (int)layers.get(listBox.getIndexes().get(i)).getPosition().getY();
                if (tempP < yP)
                    yP = tempP;
                tempP = layers.get(listBox.getIndexes().get(i)).getRightSide();
                if (tempP > rP)
                    rP = tempP;
                tempP = layers.get(listBox.getIndexes().get(i)).getBottomSide();
                if (tempP > bP)
                    bP = tempP;
            }
            propertyPanel.update(xP, yP, rP - xP, bP - yP);
        }
        else if (propertyPanel.getWorld() != null)
            propertyPanel.removeFromWorld();
        if (tool == Tool.WAND)
            for (Layer l : layers)
                l.deselect();
    }
    
    private ScrollingListener scroller = new ScrollingListener();
    private JPanel panel = WorldHandler.getInstance().getWorldCanvas();
}

class ScrollingListener implements MouseWheelListener
{
    int amount = 0;
    
    public void mouseWheelMoved(MouseWheelEvent e)
    {
        amount += e.getWheelRotation();
        e.consume();
    }
    
    public int getScroll()
    {
        int s = amount;
        amount = 0;
        return s;
    }
}